@@ -1,8 +1,8 @@
 /*****************************************************************************
  * variables.h: variables handling
  *****************************************************************************
- * Copyright (C) 2002-2004 VideoLAN
- * $Id: variables.h 7997 2004-06-18 11:35:45Z sigmunau $
+ * Copyright (C) 2002-2004 the VideoLAN team
+ * $Id: variables.h 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -87,26 +87,6 @@ struct variable_t
 #define VLC_VAR_TYPE      0x00ff
 #define VLC_VAR_FLAGS     0xff00
 
-/**
- * \defgroup var_type Variable types
- * These are the different types a vlc variable can have.
- * @{
- */
-#define VLC_VAR_VOID      0x0010
-#define VLC_VAR_BOOL      0x0020
-#define VLC_VAR_INTEGER   0x0030
-#define VLC_VAR_HOTKEY    0x0031
-#define VLC_VAR_STRING    0x0040
-#define VLC_VAR_MODULE    0x0041
-#define VLC_VAR_FILE      0x0042
-#define VLC_VAR_DIRECTORY 0x0043
-#define VLC_VAR_VARIABLE  0x0044
-#define VLC_VAR_FLOAT     0x0050
-#define VLC_VAR_TIME      0x0060
-#define VLC_VAR_ADDRESS   0x0070
-#define VLC_VAR_MUTEX     0x0080
-#define VLC_VAR_LIST      0x0090
-/**@}*/
 /** \defgroup var_flags Additive flags
  * These flags are added to the type field of the variable. Most as a result of
  * a __var_Change() call, but some may be added at creation time
