@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vlc_meta.h
  *****************************************************************************
- * Copyright (C) 2004 VideoLAN
- * $Id: vlc_meta.h 7521 2004-04-27 14:57:37Z sam $
+ * Copyright (C) 2004 the VideoLAN team
+ * $Id: vlc_meta.h 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -36,6 +36,31 @@
 #define VLC_META_SETTING            N_("Setting")
 #define VLC_META_URL                N_("URL")
 #define VLC_META_LANGUAGE           N_("Language")
+#define VLC_META_NOW_PLAYING        N_("Now Playing")
+
+#define VLC_META_CDDB_ARTIST        N_("CDDB Artist")
+#define VLC_META_CDDB_CATEGORY      N_("CDDB Category")  
+#define VLC_META_CDDB_DISCID        N_("CDDB Disc ID")
+#define VLC_META_CDDB_EXT_DATA      N_("CDDB Extended Data")
+#define VLC_META_CDDB_GENRE         N_("CDDB Genre")
+#define VLC_META_CDDB_YEAR          N_("CDDB Year")      
+#define VLC_META_CDDB_TITLE         N_("CDDB Title")
+
+#define VLC_META_CDTEXT_ARRANGER    N_("CD-Text Arranger")
+#define VLC_META_CDTEXT_COMPOSER    N_("CD-Text Composer")
+#define VLC_META_CDTEXT_DISCID      N_("CD-Text Disc ID")
+#define VLC_META_CDTEXT_GENRE       N_("CD-Text Genre")
+#define VLC_META_CDTEXT_MESSAGE     N_("CD-Text Message")
+#define VLC_META_CDTEXT_SONGWRITER  N_("CD-Text Songwriter")
+#define VLC_META_CDTEXT_PERFORMER   N_("CD-Text Performer")
+#define VLC_META_CDTEXT_TITLE       N_("CD-Text Title")
+
+#define VLC_META_ISO_APPLICATION_ID N_("ISO-9660 Application ID") 
+#define VLC_META_ISO_PREPARER       N_("ISO-9660 Preparer") 
+#define VLC_META_ISO_PUBLISHER      N_("ISO-9660 Publisher") 
+#define VLC_META_ISO_VOLUME         N_("ISO-9660 Volume") 
+#define VLC_META_ISO_VOLUMESET      N_("ISO-9660 Volume Set") 
+
 #define VLC_META_CODEC_NAME         N_("Codec Name")
 #define VLC_META_CODEC_DESCRIPTION  N_("Codec Description")
 
@@ -46,7 +71,7 @@ struct vlc_meta_t
     char    **name;
     char    **value;
 
-    /* track meta informations */
+    /* track meta information */
     int         i_track;
     vlc_meta_t  **track;
 };
@@ -84,7 +109,8 @@ static inline void vlc_meta_Delete( vlc_meta_t *m )
     free( m );
 }
 
-static inline void vlc_meta_Add( vlc_meta_t *m, char *name, char *value )
+static inline void vlc_meta_Add( vlc_meta_t *m,
+                                 const char *name, const char *value )
 {
     m->name  = (char**)realloc( m->name, sizeof(char*) * ( m->i_meta + 1 ) );
     m->name[m->i_meta] = strdup( name );
@@ -132,7 +158,7 @@ static inline void vlc_meta_Merge( vlc_meta_t *dst, vlc_meta_t *src )
     }
 }
 
-static inline char *vlc_meta_GetValue( vlc_meta_t *m, char *name )
+static inline char *vlc_meta_GetValue( vlc_meta_t *m, const char *name )
 {
     int i;
 
