@@ -1,7 +1,7 @@
 /*****************************************************************************
  * dvb.h : functions to control a DVB card under Linux with v4l2
  *****************************************************************************
- * Copyright (C) 1998-2004 VideoLAN
+ * Copyright (C) 1998-2004 the VideoLAN team
  *
  * Authors: Johan Bilien <jobi@via.ecp.fr>
  *          Jean-Paul Saman <jpsaman@saman>
@@ -54,13 +54,14 @@ typedef struct
     void *p_sys;
 } en50221_session_t;
 
-#define MAX_DEMUX 48
+#define MAX_DEMUX 256
 #define MAX_CI_SLOTS 16
 #define MAX_SESSIONS 32
+#define MAX_PROGRAMS 24
 
 struct access_sys_t
 {
-    int i_handle;
+    int i_handle, i_frontend_handle;
     demux_handle_t p_demux_handles[MAX_DEMUX];
     frontend_t *p_frontend;
     vlc_bool_t b_budget_mode;
@@ -71,9 +72,11 @@ struct access_sys_t
     vlc_bool_t pb_active_slot[MAX_CI_SLOTS];
     vlc_bool_t pb_tc_has_data[MAX_CI_SLOTS];
     en50221_session_t p_sessions[MAX_SESSIONS];
-    mtime_t i_ca_timeout, i_ca_next_event;
-    uint8_t **pp_capmts;
-    int i_nb_capmts;
+    mtime_t i_ca_timeout, i_ca_next_event, i_frontend_timeout;
+    dvbpsi_pmt_t *pp_selected_programs[MAX_PROGRAMS];
+
+    /* */
+    int i_read_once;
 };
 
 #define VIDEO0_TYPE     1
@@ -88,6 +91,7 @@ struct access_sys_t
  * Prototypes
  *****************************************************************************/
 int  E_(FrontendOpen)( access_t * );
+void E_(FrontendPoll)( access_t *p_access );
 int  E_(FrontendSet)( access_t * );
 void E_(FrontendClose)( access_t * );
 
@@ -99,10 +103,10 @@ void E_(DVRClose)( access_t * );
 
 int  E_(CAMOpen)( access_t * );
 int  E_(CAMPoll)( access_t * );
-int  E_(CAMSet)( access_t *, uint8_t **, int );
+int  E_(CAMSet)( access_t *, dvbpsi_pmt_t * );
 void E_(CAMClose)( access_t * );
 
 int E_(en50221_Poll)( access_t * );
-int E_(en50221_SetCAPMT)( access_t *, uint8_t **, int );
+int E_(en50221_SetCAPMT)( access_t *, dvbpsi_pmt_t * );
 void E_(en50221_End)( access_t * );
 
