@@ -1,8 +1,8 @@
 /*****************************************************************************
  * mms.h: MMS access plug-in
  *****************************************************************************
- * Copyright (C) 2001, 2002 VideoLAN
- * $Id: mmstu.h 6961 2004-03-05 17:34:23Z sam $
+ * Copyright (C) 2001, 2002 the VideoLAN team
+ * $Id: mmstu.h 11728 2005-07-13 18:30:46Z courmisch $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -21,23 +21,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-#if 0
-/* url: [/]host[:port][/path] */
-typedef struct url_s
-{
-    char    *psz_server_addr;
-    int     i_server_port;
-
-    char    *psz_bind_addr;
-    int     i_bind_port;
-
-    char    *psz_path;
-
-    /* private */
-    char *psz_private;
-} url_t;
-#endif
-
 #define MMS_PACKET_ANY          0
 #define MMS_PACKET_CMD          1
 #define MMS_PACKET_HEADER       2
@@ -47,36 +30,19 @@ typedef struct url_s
 
 #define MMS_CMD_HEADERSIZE  48
 
-#if 0
-#define MMS_STREAM_VIDEO    0x0001
-#define MMS_STREAM_AUDIO    0x0002
-#define MMS_STREAM_UNKNOWN  0xffff
-typedef struct mms_stream_s
-{
-    int i_id;       /* 1 -> 127 */
-    int i_cat;      /* MMS_STREAM_VIDEO, MMS_STREAM_AUDIO */
-    int i_bitrate;  /* -1 if unknown */
-    int i_selected;
-
-} mms_stream_t;
-#endif
-
 #define MMS_BUFFER_SIZE 100000
 struct access_sys_t
 {
     int                 i_proto;        /* MMS_PROTO_TCP, MMS_PROTO_UDP */
-    input_socket_t      socket_tcp;     /* TCP socket for communication with server */
-    input_socket_t      socket_udp;     /* Optional UDP socket for data(media/header packet) */
+    int                 i_handle_tcp;   /* TCP socket for communication with server */
+    int                 i_handle_udp;   /* Optional UDP socket for data(media/header packet) */
                                         /* send by server */
-    char                *psz_bind_addr; /* used by udp */
+    char                sz_bind_addr[NI_MAXNUMERICHOST]; /* used by udp */
 
-    url_t               *p_url;         /* connect to this server */
+    vlc_url_t           url;
 
-    //asf_stream_t        stream[128];    /* in asf never more than 1->127 streams */
     asf_header_t        asfh;
 
-    off_t               i_pos;          /* position of next byte to be read */
-
     /* */
     uint8_t             buffer_tcp[MMS_BUFFER_SIZE];
     int                 i_buffer_tcp;
@@ -103,9 +69,8 @@ struct access_sys_t
     size_t      i_media;
     size_t      i_media_used;
 
-    /* extracted informations */
+    /* extracted information */
     int         i_command;
-    int         i_eos;
 
     /* from 0x01 answer (not yet set) */
     char        *psz_server_version;
@@ -121,5 +86,7 @@ struct access_sys_t
     int         i_max_bit_rate;
     int         i_header_size;
 
+    /* */
+    vlc_bool_t  b_seekable;
 };
 
