@@ -1,10 +1,11 @@
 /*****************************************************************************
  * hotkeys.c: Hotkey handling for vlc
  *****************************************************************************
- * Copyright (C) 2004 VideoLAN
- * $Id: hotkeys.c 7699 2004-05-17 11:49:35Z yoann $
+ * Copyright (C) 2005 the VideoLAN team
+ * $Id: hotkeys.c 12990 2005-10-28 00:13:28Z hartman $
  *
- * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no>
+ * Authors: Sigmund Augdal Helberg <dnumgis@videolan.org>
+ *          Jean-Paul Saman <jpsaman #_at_# m2x.nl>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,11 +32,17 @@
 #include <vlc/input.h>
 #include <vlc/vout.h>
 #include <vlc/aout.h>
-#include <osd.h>
+#include <vlc_osd.h>
 
 #include "vlc_keys.h"
 
 #define BUFFER_SIZE 10
+
+#define CHANNELS_NUMBER 4
+#define VOLUME_TEXT_CHAN     p_intf->p_sys->p_channels[ 0 ]
+#define VOLUME_WIDGET_CHAN   p_intf->p_sys->p_channels[ 1 ]
+#define POSITION_TEXT_CHAN   p_intf->p_sys->p_channels[ 2 ]
+#define POSITION_WIDGET_CHAN p_intf->p_sys->p_channels[ 3 ]
 /*****************************************************************************
  * intf_sys_t: description and status of FB interface
  *****************************************************************************/
@@ -48,6 +55,8 @@ struct intf_sys_t
     int                 p_keys[ BUFFER_SIZE ]; /* buffer that contains
                                                 * keyevents */
     int                 i_size;        /* number of events in buffer */
+    int                 p_channels[ CHANNELS_NUMBER ]; /* contains registered
+                                                        * channel IDs */
     input_thread_t *    p_input;       /* pointer to input */
     vout_thread_t *     p_vout;        /* pointer to vout object */
 };
@@ -65,6 +74,9 @@ static int  ActionKeyCB( vlc_object_t *, char const *,
                          vlc_value_t, vlc_value_t, void * );
 static void PlayBookmark( intf_thread_t *, int );
 static void SetBookmark ( intf_thread_t *, int );
+static void DisplayPosition( intf_thread_t *, vout_thread_t *, input_thread_t * );
+static void DisplayVolume  ( intf_thread_t *, vout_thread_t *, audio_volume_t );
+static void ClearChannels  ( intf_thread_t *, vout_thread_t * );
 
 /*****************************************************************************
  * Module descriptor
@@ -141,6 +153,7 @@ static void Close( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
 
+    var_DelCallback( p_intf->p_vlc, "key-pressed", KeyEvent, p_intf );
     if( p_intf->p_sys->p_input )
     {
         vlc_object_release( p_intf->p_sys->p_input );
@@ -158,9 +171,10 @@ static void Close( vlc_object_t *p_this )
  *****************************************************************************/
 static void Run( intf_thread_t *p_intf )
 {
-    playlist_t *p_playlist;
-    input_thread_t *p_input;
+    playlist_t *p_playlist = NULL;
+    input_thread_t *p_input = NULL;
     vout_thread_t *p_vout = NULL;
+    vout_thread_t *p_last_vout = NULL;
     struct hotkey *p_hotkeys = p_intf->p_vlc->p_hotkeys;
     vlc_value_t val;
     int i;
@@ -180,6 +194,7 @@ static void Run( intf_thread_t *p_intf )
     while( !p_intf->b_die )
     {
         int i_key, i_action;
+        int i_times = 0;
 
         /* Sleep a bit */
         msleep( INTF_IDLE_SLEEP );
@@ -198,6 +213,7 @@ static void Run( intf_thread_t *p_intf )
         p_input = p_intf->p_sys->p_input;
 
         /* Update the vout */
+        p_last_vout = p_intf->p_sys->p_vout;
         if( p_vout == NULL )
         {
             p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT, FIND_ANYWHERE );
@@ -210,14 +226,26 @@ static void Run( intf_thread_t *p_intf )
             p_intf->p_sys->p_vout = NULL;
         }
 
+        /* Register OSD channels */
+        if( p_vout && p_vout != p_last_vout )
+        {
+            for( i = 0; i < CHANNELS_NUMBER; i++ )
+            {
+                spu_Control( p_vout->p_spu, SPU_CHANNEL_REGISTER,
+                             &p_intf->p_sys->p_channels[ i ] );
+            }
+        }
+
         /* Find action triggered by hotkey */
         i_action = 0;
         i_key = GetKey( p_intf );
-        for( i = 0; p_hotkeys[i].psz_action != NULL; i++ )
+        for( i = 0; i_key != -1 && p_hotkeys[i].psz_action != NULL; i++ )
         {
             if( p_hotkeys[i].i_key == i_key )
             {
-                 i_action = p_hotkeys[i].i_action;
+                i_action = p_hotkeys[i].i_action;
+                i_times  = p_hotkeys[i].i_times; /* times key pressed within max. delta time */
+                p_hotkeys[i].i_times = 0;
             }
         }
 
@@ -230,73 +258,94 @@ static void Run( intf_thread_t *p_intf )
 
         if( i_action == ACTIONID_QUIT )
         {
+            p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                    FIND_ANYWHERE );
+            if( p_playlist )
+            {
+                playlist_Stop( p_playlist );
+                vlc_object_release( p_playlist );
+            }
+            /* Playlist is stopped now kill vlc. */
             p_intf->p_vlc->b_die = VLC_TRUE;
-            vout_OSDMessage( p_intf, _( "Quit" ) );
+            ClearChannels( p_intf, p_vout );
+            vout_OSDMessage( p_intf, DEFAULT_CHAN, _( "Quit" ) );
             continue;
         }
         else if( i_action == ACTIONID_VOL_UP )
         {
             audio_volume_t i_newvol;
             aout_VolumeUp( p_intf, 1, &i_newvol );
-            vout_OSDMessage( p_intf, "Vol %d%%", i_newvol*100/AOUT_VOLUME_MAX );
+            DisplayVolume( p_intf, p_vout, i_newvol );
         }
         else if( i_action == ACTIONID_VOL_DOWN )
         {
             audio_volume_t i_newvol;
             aout_VolumeDown( p_intf, 1, &i_newvol );
-            vout_OSDMessage( p_intf, "Vol %d%%", i_newvol*100/AOUT_VOLUME_MAX);
-        }
-        else if( i_action == ACTIONID_SUBDELAY_DOWN )
-        {
-            int i_delay;
-            if( input_Control( p_input, INPUT_GET_SUBDELAY, &i_delay ) ==
-                VLC_SUCCESS )
-            {
-                i_delay--;
-                input_Control( p_input, INPUT_SET_SUBDELAY, i_delay );
-                vout_OSDMessage( p_intf, "Subtitle delay %i ms", i_delay*100);
-            }
-        }
-        else if( i_action == ACTIONID_SUBDELAY_UP )
-        {
-            int i_delay;
-            if( input_Control( p_input, INPUT_GET_SUBDELAY, &i_delay ) ==
-                VLC_SUCCESS )
-            {
-                i_delay++;
-                input_Control( p_input, INPUT_SET_SUBDELAY, i_delay );
-                vout_OSDMessage( p_intf, "Subtitle delay %i ms", i_delay*100);
-            }
+            DisplayVolume( p_intf, p_vout, i_newvol );
         }
         else if( i_action == ACTIONID_VOL_MUTE )
         {
             audio_volume_t i_newvol = -1;
             aout_VolumeMute( p_intf, &i_newvol );
-            if( i_newvol == 0 )
+            if( p_vout )
             {
-                vout_OSDMessage( p_intf, _( "Mute" ) );
-            }
-            else
-            {
-                vout_OSDMessage( p_intf, "Vol %d%%",
-                                 i_newvol * 100 / AOUT_VOLUME_MAX );
+                if( i_newvol == 0 )
+                {
+                    ClearChannels( p_intf, p_vout );
+                    vout_OSDIcon( VLC_OBJECT( p_intf ), DEFAULT_CHAN,
+                                  OSD_MUTE_ICON );
+                }
+                else
+                {
+                    DisplayVolume( p_intf, p_vout, i_newvol );
+                }
             }
         }
-        else if( i_action == ACTIONID_FULLSCREEN && p_vout )
-        {
-            var_Get( p_vout, "fullscreen", &val );
-            var_Set( p_vout, "fullscreen", (vlc_value_t)!val.b_bool );
-        }
-        else if( i_action == ACTIONID_PLAY )
+        else if( i_action == ACTIONID_INTF_SHOW )
         {
+            val.b_bool = VLC_TRUE;
             p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
                                           FIND_ANYWHERE );
             if( p_playlist )
             {
-                playlist_Play( p_playlist );
+                var_Set( p_playlist, "intf-show", val );
                 vlc_object_release( p_playlist );
             }
         }
+        else if( i_action == ACTIONID_INTF_HIDE )
+        {
+            val.b_bool = VLC_FALSE;
+            p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                          FIND_ANYWHERE );
+            if( p_playlist )
+            {
+                var_Set( p_playlist, "intf-show", val );
+                vlc_object_release( p_playlist );
+            }
+        }
+        else if( i_action == ACTIONID_SNAPSHOT )
+        {
+            if( p_vout ) vout_Control( p_vout, VOUT_SNAPSHOT );
+        }
+        else if( i_action == ACTIONID_FULLSCREEN )
+        {
+            if( p_vout )
+            {
+                var_Get( p_vout, "fullscreen", &val ); val.b_bool = !val.b_bool;
+                var_Set( p_vout, "fullscreen", val );
+            }
+            else
+            {
+                p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                          FIND_ANYWHERE );
+                if( p_playlist )
+                {
+                    var_Get( p_playlist, "fullscreen", &val ); val.b_bool = !val.b_bool;
+                    var_Set( p_playlist, "fullscreen", val );
+                    vlc_object_release( p_playlist );
+                }
+            }
+        }
         else if( i_action == ACTIONID_PLAY_PAUSE )
         {
             val.i_int = PLAYING_S;
@@ -306,7 +355,9 @@ static void Run( intf_thread_t *p_intf )
             }
             if( p_input && val.i_int != PAUSE_S )
             {
-                vout_OSDMessage( p_intf, _( "Pause" ) );
+                ClearChannels( p_intf, p_vout );
+                vout_OSDIcon( VLC_OBJECT( p_intf ), DEFAULT_CHAN,
+                              OSD_PAUSE_ICON );
                 val.i_int = PAUSE_S;
                 var_Set( p_input, "state", val );
             }
@@ -316,7 +367,9 @@ static void Run( intf_thread_t *p_intf )
                                               FIND_ANYWHERE );
                 if( p_playlist )
                 {
-                    vout_OSDMessage( p_intf, _( "Play" ) );
+                    ClearChannels( p_intf, p_vout );
+                    vout_OSDIcon( VLC_OBJECT( p_intf ), DEFAULT_CHAN,
+                                  OSD_PLAY_ICON );
                     playlist_Play( p_playlist );
                     vlc_object_release( p_playlist );
                 }
@@ -324,47 +377,153 @@ static void Run( intf_thread_t *p_intf )
         }
         else if( p_input )
         {
+            /* FIXME --fenrir
+             * How to get a valid value ?
+             * That's not that easy with some special stream
+             */
+            vlc_bool_t b_seekable = VLC_TRUE;
+
             if( i_action == ACTIONID_PAUSE )
             {
-                vout_OSDMessage( p_intf, _( "Pause" ) );
+                ClearChannels( p_intf, p_vout );
+                vout_OSDIcon( VLC_OBJECT( p_intf ), DEFAULT_CHAN,
+                              OSD_PAUSE_ICON );
                 val.i_int = PAUSE_S;
                 var_Set( p_input, "state", val );
             }
-            else if( i_action == ACTIONID_JUMP_BACKWARD_10SEC )
+            else if( i_action == ACTIONID_JUMP_BACKWARD_3SEC && b_seekable )
             {
-                vout_OSDMessage( p_intf, _( "Jump -10 seconds" ) );
-                val.i_time = -10000000;
+                val.i_time = (-3000000L * ((mtime_t)(1 << i_times)));
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
             }
-            else if( i_action == ACTIONID_JUMP_FORWARD_10SEC )
+            else if( i_action == ACTIONID_JUMP_FORWARD_3SEC && b_seekable )
             {
-                vout_OSDMessage( p_intf, _( "Jump +10 seconds" ) );
-                val.i_time = 10000000;
+                val.i_time = (3000000L * ((mtime_t)(1 << i_times)));
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
             }
-            else if( i_action == ACTIONID_JUMP_BACKWARD_1MIN )
+            else if( i_action == ACTIONID_JUMP_BACKWARD_10SEC && b_seekable )
             {
-                vout_OSDMessage( p_intf, _( "Jump -1 minute" ) );
-                val.i_time = -60000000;
+                val.i_time = (-10000000L * ((mtime_t)(1 << i_times)));
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
             }
-            else if( i_action == ACTIONID_JUMP_FORWARD_1MIN )
+            else if( i_action == ACTIONID_JUMP_FORWARD_10SEC && b_seekable )
             {
-                vout_OSDMessage( p_intf, _( "Jump +1 minute" ) );
-                val.i_time = 60000000;
+                val.i_time = (10000000L * ((mtime_t)(1 << i_times)));
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
             }
-            else if( i_action == ACTIONID_JUMP_BACKWARD_5MIN )
+            else if( i_action == ACTIONID_JUMP_BACKWARD_1MIN && b_seekable )
             {
-                vout_OSDMessage( p_intf, _( "Jump -5 minutes" ) );
-                val.i_time = -300000000;
+                val.i_time = (-60000000L * ((mtime_t)(1 << i_times)));
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
             }
-            else if( i_action == ACTIONID_JUMP_FORWARD_5MIN )
+            else if( i_action == ACTIONID_JUMP_FORWARD_1MIN && b_seekable )
             {
-                vout_OSDMessage( p_intf, _( "Jump +5 minutes" ) );
-                val.i_time = 300000000;
+                val.i_time = (60000000L * ((mtime_t)(1 << i_times)));
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
+            }
+            else if( i_action == ACTIONID_JUMP_BACKWARD_5MIN && b_seekable )
+            {
+                val.i_time = (-300000000L * ((mtime_t)(1 << i_times)));
+                var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
+            }
+            else if( i_action == ACTIONID_JUMP_FORWARD_5MIN && b_seekable )
+            {
+                val.i_time = (300000000L * ((mtime_t)(1 << i_times)));
+                var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
+            }
+            else if( i_action == ACTIONID_AUDIO_TRACK )
+            {
+                vlc_value_t val, list, list2;
+                int i_count, i;
+                var_Get( p_input, "audio-es", &val );
+                var_Change( p_input, "audio-es", VLC_VAR_GETCHOICES,
+                            &list, &list2 );
+                i_count = list.p_list->i_count;
+                if( i_count <= 1 )
+                {
+                    continue;
+                }
+                for( i = 0; i < i_count; i++ )
+                {
+                    if( val.i_int == list.p_list->p_values[i].i_int )
+                    {
+                        break;
+                    }
+                }
+                /* value of audio-es was not in choices list */
+                if( i == i_count )
+                {
+                    msg_Warn( p_input,
+                              "invalid current audio track, selecting 0" );
+                    var_Set( p_input, "audio-es",
+                             list.p_list->p_values[0] );
+                    i = 0;
+                }
+                else if( i == i_count - 1 )
+                {
+                    var_Set( p_input, "audio-es",
+                             list.p_list->p_values[1] );
+                    i = 1;
+                }
+                else
+                {
+                    var_Set( p_input, "audio-es",
+                             list.p_list->p_values[i+1] );
+                    i++;
+                }
+                vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN,
+                                 _("Audio track: %s"),
+                                 list2.p_list->p_values[i].psz_string );
+            }
+            else if( i_action == ACTIONID_SUBTITLE_TRACK )
+            {
+                vlc_value_t val, list, list2;
+                int i_count, i;
+                var_Get( p_input, "spu-es", &val );
+
+                var_Change( p_input, "spu-es", VLC_VAR_GETCHOICES,
+                            &list, &list2 );
+                i_count = list.p_list->i_count;
+                if( i_count <= 1 )
+                {
+                    vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN, _("Subtitle track: %s"), _("N/A") );
+                    continue;
+                }
+                for( i = 0; i < i_count; i++ )
+                {
+                    if( val.i_int == list.p_list->p_values[i].i_int )
+                    {
+                        break;
+                    }
+                }
+                /* value of spu-es was not in choices list */
+                if( i == i_count )
+                {
+                    msg_Warn( p_input, "invalid current subtitle track, selecting 0" );
+                    var_Set( p_input, "spu-es", list.p_list->p_values[0] );
+                    i = 0;
+                }
+                else if( i == i_count - 1 )
+                {
+                    var_Set( p_input, "spu-es", list.p_list->p_values[0] );
+                    i = 0;
+                }
+                else
+                {
+                    var_Set( p_input, "spu-es", list.p_list->p_values[i+1] );
+                    i = i + 1;
+                }
+                vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN,
+                                 _("Subtitle track: %s"),
+                                 list2.p_list->p_values[i].psz_string );
             }
             else if( i_action == ACTIONID_NEXT )
             {
@@ -372,6 +531,7 @@ static void Run( intf_thread_t *p_intf )
                                               FIND_ANYWHERE );
                 if( p_playlist )
                 {
+                    vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN, _("Next") );
                     playlist_Next( p_playlist );
                     vlc_object_release( p_playlist );
                 }
@@ -382,6 +542,7 @@ static void Run( intf_thread_t *p_intf )
                                               FIND_ANYWHERE );
                 if( p_playlist )
                 {
+                    vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN, _("Previous") );
                     playlist_Prev( p_playlist );
                     vlc_object_release( p_playlist );
                 }
@@ -398,36 +559,21 @@ static void Run( intf_thread_t *p_intf )
             }
             else if( i_action == ACTIONID_FASTER )
             {
-                vlc_value_t val; val.b_bool = VLC_TRUE;
+                vlc_value_t val;
+                val.b_bool = VLC_TRUE;
                 var_Set( p_input, "rate-faster", val );
+                vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN, _("Faster") );
             }
             else if( i_action == ACTIONID_SLOWER )
             {
-                vlc_value_t val; val.b_bool = VLC_TRUE;
+                vlc_value_t val;
+                val.b_bool = VLC_TRUE;
                 var_Set( p_input, "rate-slower", val );
+                vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN, _("Slower") );
             }
-            else if( i_action == ACTIONID_POSITION )
+            else if( i_action == ACTIONID_POSITION && b_seekable )
             {
-                char psz_duration[MSTRTIME_MAX_SIZE];
-                char psz_time[MSTRTIME_MAX_SIZE];
-                vlc_value_t time;
-                mtime_t i_seconds;
-
-                var_Get( p_input, "time", &time );
-                i_seconds = time.i_time / 1000000;
-                secstotimestr ( psz_time, i_seconds );
-
-                var_Get( p_input, "length", &time );
-                if( time.i_time > 0 )
-                {
-                    secstotimestr( psz_duration, time.i_time / 1000000 );
-                    vout_OSDMessage( p_input, "%s / %s",
-                                     psz_time, psz_duration );
-                }
-                else if( i_seconds > 0 )
-                {
-                    vout_OSDMessage( p_input, psz_time );
-                }
+                DisplayPosition( p_intf, p_vout, p_input );
             }
             else if( i_action >= ACTIONID_PLAY_BOOKMARK1 &&
                      i_action <= ACTIONID_PLAY_BOOKMARK10 )
@@ -439,6 +585,101 @@ static void Run( intf_thread_t *p_intf )
             {
                 SetBookmark( p_intf, i_action - ACTIONID_SET_BOOKMARK1 + 1 );
             }
+            /* Only makes sense with DVD */
+            else if( i_action == ACTIONID_TITLE_PREV )
+            {
+                val.b_bool = VLC_TRUE;
+                var_Set( p_input, "prev-title", val );
+            }
+            else if( i_action == ACTIONID_TITLE_NEXT )
+            {
+                val.b_bool = VLC_TRUE;
+                var_Set( p_input, "next-title", val );
+            }
+            else if( i_action == ACTIONID_CHAPTER_PREV )
+            {
+                val.b_bool = VLC_TRUE;
+                var_Set( p_input, "prev-chapter", val );
+            }
+            else if( i_action == ACTIONID_CHAPTER_NEXT )
+            {
+                val.b_bool = VLC_TRUE;
+                var_Set( p_input, "next-chapter", val );
+            }
+            else if( i_action == ACTIONID_DISC_MENU )
+            {
+                vlc_value_t val; val.i_int = 2;
+msg_Dbg( p_input, "set dvdmenu" );
+                var_Set( p_input, "title  0", val);
+            }
+            else if( i_action == ACTIONID_SUBDELAY_DOWN )
+            {
+                int64_t i_delay = var_GetTime( p_input, "spu-delay" );
+
+                i_delay -= 50000;    /* 50 ms */
+
+                var_SetTime( p_input, "spu-delay", i_delay );
+                ClearChannels( p_intf, p_vout );
+                vout_OSDMessage( p_intf, DEFAULT_CHAN, "Subtitle delay %i ms",
+                                 (int)(i_delay/1000) );
+            }
+            else if( i_action == ACTIONID_SUBDELAY_UP )
+            {
+                int64_t i_delay = var_GetTime( p_input, "spu-delay" );
+
+                i_delay += 50000;    /* 50 ms */
+
+                var_SetTime( p_input, "spu-delay", i_delay );
+                ClearChannels( p_intf, p_vout );
+                vout_OSDMessage( p_intf, DEFAULT_CHAN, "Subtitle delay %i ms",
+                                 (int)(i_delay/1000) );
+            }
+            else if( i_action == ACTIONID_AUDIODELAY_DOWN )
+            {
+                int64_t i_delay = var_GetTime( p_input, "audio-delay" );
+
+                i_delay -= 50000;    /* 50 ms */
+
+                var_SetTime( p_input, "audio-delay", i_delay );
+                ClearChannels( p_intf, p_vout );
+                vout_OSDMessage( p_intf, DEFAULT_CHAN, "Audio delay %i ms",
+                                 (int)(i_delay/1000) );
+            }
+            else if( i_action == ACTIONID_AUDIODELAY_UP )
+            {
+                int64_t i_delay = var_GetTime( p_input, "audio-delay" );
+
+                i_delay += 50000;    /* 50 ms */
+
+                var_SetTime( p_input, "audio-delay", i_delay );
+                ClearChannels( p_intf, p_vout );
+                vout_OSDMessage( p_intf, DEFAULT_CHAN, "Audio delay %i ms",
+                                 (int)(i_delay/1000) );
+            }
+            else if( i_action == ACTIONID_PLAY )
+            {
+                p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                              FIND_ANYWHERE );
+                if( p_playlist )
+                {
+                    var_Get( p_input, "rate", &val );
+                    msg_Dbg( p_input, "rate %d", val.i_int );
+                    if( val.i_int != INPUT_RATE_DEFAULT )
+                    {
+                        /* Return to normal speed */
+                        val.i_int = INPUT_RATE_DEFAULT;
+                        var_Set( p_input, "rate", val );
+                    }
+                    else
+                    {
+                        ClearChannels( p_intf, p_vout );
+                        vout_OSDIcon( VLC_OBJECT( p_intf ), DEFAULT_CHAN,
+                                      OSD_PAUSE_ICON );
+                        playlist_Play( p_playlist );
+                    }
+                    vlc_object_release( p_playlist );
+                }
+            }
         }
     }
 }
@@ -494,6 +735,7 @@ static int ActionKeyCB( vlc_object_t *p_this, char const *psz_var,
 {
     vlc_t *p_vlc = (vlc_t *)p_this;
     struct hotkey *p_hotkeys = p_vlc->p_hotkeys;
+    mtime_t i_date;
     int i;
 
     for( i = 0; p_hotkeys[i].psz_action != NULL; i++ )
@@ -501,6 +743,14 @@ static int ActionKeyCB( vlc_object_t *p_this, char const *psz_var,
         if( !strcmp( p_hotkeys[i].psz_action, psz_var ) )
         {
             p_hotkeys[i].i_key = newval.i_int;
+            /* do hotkey accounting */
+            i_date = mdate();
+            if( (p_hotkeys[i].i_delta_date > 0) &&
+                (p_hotkeys[i].i_delta_date <= (i_date - p_hotkeys[i].i_last_date) ) )
+                p_hotkeys[i].i_times = 0;
+            else
+                p_hotkeys[i].i_times++;
+            p_hotkeys[i].i_last_date = i_date;
         }
     }
 
@@ -537,7 +787,6 @@ static void PlayBookmark( intf_thread_t *p_intf, int i_num )
 
 static void SetBookmark( intf_thread_t *p_intf, int i_num )
 {
-    vlc_value_t val;
     playlist_t *p_playlist =
         vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
     if( p_playlist )
@@ -546,10 +795,88 @@ static void SetBookmark( intf_thread_t *p_intf, int i_num )
         sprintf( psz_bookmark_name, "bookmark%i", i_num );
         var_Create( p_intf, psz_bookmark_name,
                     VLC_VAR_STRING|VLC_VAR_DOINHERIT );
-        val.psz_string = strdup( p_playlist->pp_items[p_playlist->i_index]->input.psz_uri );
-        var_Set( p_intf, psz_bookmark_name, val );
-        msg_Info( p_intf, "setting playlist bookmark %i to %s", i_num,
-                  val.psz_string );
+        if( p_playlist->status.p_item )
+        {
+            config_PutPsz( p_intf, psz_bookmark_name, 
+                           p_playlist->status.p_item->input.psz_uri);
+            msg_Info( p_intf, "setting playlist bookmark %i to %s", i_num,
+                           p_playlist->status.p_item->input.psz_uri);
+            config_SaveConfigFile( p_intf, "hotkeys" );
+        }
         vlc_object_release( p_playlist );
     }
 }
+
+static void DisplayPosition( intf_thread_t *p_intf, vout_thread_t *p_vout,
+                             input_thread_t *p_input )
+{
+    char psz_duration[MSTRTIME_MAX_SIZE];
+    char psz_time[MSTRTIME_MAX_SIZE];
+    vlc_value_t time, pos;
+    mtime_t i_seconds;
+
+    if( p_vout == NULL )
+    {
+        return;
+    }
+    ClearChannels( p_intf, p_vout );
+
+    var_Get( p_input, "time", &time );
+    i_seconds = time.i_time / 1000000;
+    secstotimestr ( psz_time, i_seconds );
+
+    var_Get( p_input, "length", &time );
+    if( time.i_time > 0 )
+    {
+        secstotimestr( psz_duration, time.i_time / 1000000 );
+        vout_OSDMessage( p_input, POSITION_TEXT_CHAN, "%s / %s",
+                         psz_time, psz_duration );
+    }
+    else if( i_seconds > 0 )
+    {
+        vout_OSDMessage( p_input, POSITION_TEXT_CHAN, psz_time );
+    }
+
+    if( !p_vout->p_parent_intf || p_vout->b_fullscreen )
+    {
+        var_Get( p_input, "position", &pos );
+        vout_OSDSlider( VLC_OBJECT( p_input ), POSITION_WIDGET_CHAN,
+                        pos.f_float * 100, OSD_HOR_SLIDER );
+    }
+}
+
+static void DisplayVolume( intf_thread_t *p_intf, vout_thread_t *p_vout,
+                           audio_volume_t i_vol )
+{
+    if( p_vout == NULL )
+    {
+        return;
+    }
+    ClearChannels( p_intf, p_vout );
+
+    if( !p_vout->p_parent_intf || p_vout->b_fullscreen )
+    {
+        vout_OSDSlider( VLC_OBJECT( p_vout ), VOLUME_WIDGET_CHAN,
+            i_vol*100/AOUT_VOLUME_MAX, OSD_VERT_SLIDER );
+    }
+    else
+    {
+        vout_OSDMessage( p_vout, VOLUME_TEXT_CHAN, "Volume %d%%",
+                         i_vol*400/AOUT_VOLUME_MAX );
+    }
+}
+
+static void ClearChannels( intf_thread_t *p_intf, vout_thread_t *p_vout )
+{
+    int i;
+
+    if( p_vout )
+    {
+        spu_Control( p_vout->p_spu, SPU_CHANNEL_CLEAR, DEFAULT_CHAN );
+        for( i = 0; i < CHANNELS_NUMBER; i++ )
+        {
+            spu_Control( p_vout->p_spu, SPU_CHANNEL_CLEAR,
+                         p_intf->p_sys->p_channels[ i ] );
+        }
+    }
+}
