@@ -1,8 +1,8 @@
 /*****************************************************************************
  * InterfaceWindow.cpp: beos interface
  *****************************************************************************
- * Copyright (C) 1999, 2000, 2001 VideoLAN
- * $Id: InterfaceWindow.cpp 8652 2004-09-06 16:46:25Z titer $
+ * Copyright (C) 1999, 2000, 2001 the VideoLAN team
+ * $Id: InterfaceWindow.cpp 12687 2005-09-26 15:03:39Z titer $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -174,6 +174,14 @@ collect_folder_contents( BDirectory& dir, BList& list, bool& deep, bool& asked,
     }
 }
 
+static int PlaylistChanged( vlc_object_t *p_this, const char * psz_variable,
+                            vlc_value_t old_val, vlc_value_t new_val,
+                            void * param )
+{
+    InterfaceWindow * w = (InterfaceWindow *) param;
+    w->UpdatePlaylist();
+    return VLC_SUCCESS;
+}
 
 /*****************************************************************************
  * InterfaceWindow
@@ -183,16 +191,25 @@ InterfaceWindow::InterfaceWindow( intf_thread_t * _p_intf, BRect frame,
                                   const char * name )
     : BWindow( frame, name, B_TITLED_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL,
                B_NOT_ZOOMABLE | B_WILL_ACCEPT_FIRST_CLICK | B_ASYNCHRONOUS_CONTROLS ),
-      
+
       /* Initializations */
       p_intf( _p_intf ),
       p_input( NULL ),
       p_playlist( NULL ),
-      
+
       fFilePanel( NULL ),
       fLastUpdateTime( system_time() ),
       fSettings( new BMessage( 'sett' ) )
 {
+    p_playlist = (playlist_t *)
+        vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
+
+    var_AddCallback( p_playlist, "intf-change", PlaylistChanged, this );
+    var_AddCallback( p_playlist, "item-change", PlaylistChanged, this );
+    var_AddCallback( p_playlist, "item-append", PlaylistChanged, this );
+    var_AddCallback( p_playlist, "item-deleted", PlaylistChanged, this );
+    var_AddCallback( p_playlist, "playlist-current", PlaylistChanged, this );
+
     char psz_tmp[1024];
 #define ADD_ELLIPSIS( a ) \
     memset( psz_tmp, 0, 1024 ); \
@@ -374,7 +391,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
         case B_ABOUT_REQUESTED:
         {
             BAlert * alert;
-            
+
             alert = new BAlert( "VLC media player" VERSION,
                                 "VLC media player" VERSION " (BeOS interface)\n\n"
                                 "The VideoLAN team <videolan@videolan.org>\n"
@@ -416,7 +433,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
                     playlist_Add( p_playlist, psz_uri, psz_device,
                                   PLAYLIST_APPEND | PLAYLIST_GO, PLAYLIST_END );
                 }
-                _UpdatePlaylist();
+                UpdatePlaylist();
             }
             break;
 
@@ -439,27 +456,27 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
             }
             p_mediaControl->SetStatus(-1, INPUT_RATE_DEFAULT);
             break;
-    
+
         case START_PLAYBACK:
-            /*  starts playing in normal mode */
-    
         case PAUSE_PLAYBACK:
+        {
+            vlc_value_t val;
+            val.i_int = PLAYING_S;
             if( p_input )
             {
-                if( var_GetInteger( p_input, "state" ) == PAUSE_S )
-                {
-                    if( p_playlist )
-                    {
-                        playlist_Play( p_playlist );
-                    }
-                }
-                else
-                {
-                    var_SetInteger( p_input, "state", PAUSE_S );
-                }
+                var_Get( p_input, "state", &val );
+            }
+            if( p_input && val.i_int != PAUSE_S )
+            {
+                val.i_int = PAUSE_S;
+                var_Set( p_input, "state", val );
+            }
+            else
+            {
+                playlist_Play( p_playlist );
             }
             break;
-    
+        }
         case HEIGHTH_PLAY:
             if( p_input )
             {
@@ -516,11 +533,11 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
         case VOLUME_CHG:
             aout_VolumeSet( p_intf, p_mediaControl->GetVolume() );
             break;
-    
+
         case VOLUME_MUTE:
             aout_VolumeMute( p_intf, NULL );
             break;
-    
+
         case SELECT_CHANNEL:
         {
             int32 channel;
@@ -537,7 +554,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
             }
             break;
         }
-    
+
         case PREV_TITLE:
             if( p_input )
             {
@@ -708,17 +725,13 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
             {
                 break;
             }
-            
+
             vlc_bool_t b_remove = ( p_message->WasDropped() &&
-                                    !( modifiers() & B_SHIFT_KEY ) ); 
+                                    !( modifiers() & B_SHIFT_KEY ) );
 
             if( b_remove && p_playlist )
             {
-                /* Empty playlist */
-                while( p_playlist->i_size > 0 )
-                {
-                    playlist_Delete( p_playlist, 0 );
-                }
+                playlist_Clear( p_playlist );
             }
 
             entry_ref ref;
@@ -735,7 +748,7 @@ void InterfaceWindow::MessageReceived( BMessage * p_message )
                 }
             }
 
-            _UpdatePlaylist();
+            UpdatePlaylist();
             break;
         }
 
@@ -785,7 +798,7 @@ bool InterfaceWindow::QuitRequested()
     p_mediaControl->SetStatus(-1, INPUT_RATE_DEFAULT);
 
      _StoreSettings();
-   
+
     p_intf->b_die = 1;
 
     return( true );
@@ -796,12 +809,6 @@ bool InterfaceWindow::QuitRequested()
  *****************************************************************************/
 void InterfaceWindow::UpdateInterface()
 {
-    /* Manage the input part */
-    if( !p_playlist )
-    {
-        p_playlist = (playlist_t *)
-            vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
-    }
     if( !p_input )
     {
         p_input = (input_thread_t *)
@@ -818,14 +825,25 @@ void InterfaceWindow::UpdateInterface()
     {
         return;
     }
-    
+
+    if( b_playlist_update )
+    {
+        if( fPlaylistWindow->Lock() )
+        {
+            fPlaylistWindow->UpdatePlaylist( true );
+            fPlaylistWindow->Unlock();
+            b_playlist_update = false;
+        }
+        p_mediaControl->SetEnabled( p_playlist->i_size );
+    }
+
     if( p_input )
     {
         vlc_value_t val;
         p_mediaControl->SetEnabled( true );
         bool hasTitles   = !var_Get( p_input, "title", &val );
         bool hasChapters = !var_Get( p_input, "chapter", &val );
-        p_mediaControl->SetStatus( var_GetInteger( p_input, "state" ), 
+        p_mediaControl->SetStatus( var_GetInteger( p_input, "state" ),
                                    var_GetInteger( p_input, "rate" ) );
         var_Get( p_input, "position", &val );
         p_mediaControl->SetProgress( val.f_float );
@@ -844,24 +862,17 @@ void InterfaceWindow::UpdateInterface()
         aout_VolumeGet( p_intf, &i_volume );
         p_mediaControl->SetAudioEnabled( true );
         p_mediaControl->SetMuted( i_volume );
-
-        // update playlist as well
-        if( fPlaylistWindow->LockWithTimeout( INTERFACE_LOCKING_TIMEOUT ) == B_OK )
-        {
-            fPlaylistWindow->UpdatePlaylist();
-            fPlaylistWindow->Unlock();
-        }
     }
     else
     {
         p_mediaControl->SetAudioEnabled( false );
 
         _SetMenusEnabled( false );
-        
-        if( !p_playlist || p_playlist->i_size <= 0 )
+
+        if( !playlist_IsEmpty( p_playlist ) )
         {
             p_mediaControl->SetProgress( 0 );
-            
+
 #if 0
             // enable/disable skip buttons
             bool canSkipPrev;
@@ -875,11 +886,20 @@ void InterfaceWindow::UpdateInterface()
             p_mediaControl->SetEnabled( false );
         }
     }
-    
+
     Unlock();
     fLastUpdateTime = system_time();
 }
 
+/*****************************************************************************
+ * InterfaceWindow::UpdatePlaylist
+ *****************************************************************************/
+void
+InterfaceWindow::UpdatePlaylist()
+{
+    b_playlist_update = true;
+}
+
 /*****************************************************************************
  * InterfaceWindow::IsStopped
  *****************************************************************************/
@@ -889,20 +909,6 @@ InterfaceWindow::IsStopped() const
     return (system_time() - fLastUpdateTime > INTERFACE_UPDATE_TIMEOUT);
 }
 
-/*****************************************************************************
- * InterfaceWindow::_UpdatePlaylist
- *****************************************************************************/
-void
-InterfaceWindow::_UpdatePlaylist()
-{
-    if( fPlaylistWindow->Lock() )
-    {
-        fPlaylistWindow->UpdatePlaylist( true );
-        fPlaylistWindow->Unlock();
-    }
-    p_mediaControl->SetEnabled( p_playlist->i_size );
-}
-
 /*****************************************************************************
  * InterfaceWindow::_SetMenusEnabled
  *****************************************************************************/
@@ -949,33 +955,33 @@ void
 InterfaceWindow::_UpdateSpeedMenu( int rate )
 {
     BMenuItem * toMark = NULL;
-    
+
     switch( rate )
     {
         case ( INPUT_RATE_DEFAULT * 8 ):
             toMark = fHeighthMI;
             break;
-            
+
         case ( INPUT_RATE_DEFAULT * 4 ):
             toMark = fQuarterMI;
             break;
-            
+
         case ( INPUT_RATE_DEFAULT * 2 ):
             toMark = fHalfMI;
             break;
-            
+
         case ( INPUT_RATE_DEFAULT ):
             toMark = fNormalMI;
             break;
-            
+
         case ( INPUT_RATE_DEFAULT / 2 ):
             toMark = fTwiceMI;
             break;
-            
+
         case ( INPUT_RATE_DEFAULT / 4 ):
             toMark = fFourMI;
             break;
-            
+
         case ( INPUT_RATE_DEFAULT / 8 ):
             toMark = fHeightMI;
             break;
@@ -1069,7 +1075,7 @@ InterfaceWindow::_RestoreSettings()
             frame.bottom = frame.top + fPreferencesWindow->Frame().Height();
             set_window_pos( fPreferencesWindow, frame );
         }
-        
+
         bool showing;
         if ( fSettings->FindBool( "playlist showing", &showing ) == B_OK )
             launch_window( fPlaylistWindow, showing );
@@ -1224,11 +1230,11 @@ void LanguageMenu::AttachedToWindow()
     {
         return;
     }
-    
+
     vlc_value_t val_list, text_list;
     BMessage * message;
     int i_current;
-    
+
     i_current = var_GetInteger( p_input, psz_variable );
     var_Change( p_input, psz_variable, VLC_VAR_GETLIST, &val_list, &text_list );
     for( int i = 0; i < val_list.p_list->i_count; i++ )
@@ -1243,9 +1249,9 @@ void LanguageMenu::AttachedToWindow()
         AddItem( item );
     }
     var_Change( p_input, psz_variable, VLC_VAR_FREELIST, &val_list, &text_list );
-    
+
     vlc_object_release( p_input );
-    
+
     BMenu::AttachedToWindow();
 }
 
@@ -1291,7 +1297,7 @@ void TitleMenu::AttachedToWindow()
         vlc_value_t val_list, text_list;
         var_Change( p_input, "title", VLC_VAR_GETCHOICES,
                     &val_list, &text_list );
-        
+
         for( int i = 0; i < val_list.p_list->i_count; i++ )
         {
             message = new BMessage( TOGGLE_TITLE );
@@ -1308,6 +1314,7 @@ void TitleMenu::AttachedToWindow()
         var_Change( p_input, "title", VLC_VAR_FREELIST,
                     &val_list, &text_list );
     }
+    vlc_object_release( p_input );
     BMenu::AttachedToWindow();
 }
 
@@ -1354,7 +1361,7 @@ void ChapterMenu::AttachedToWindow()
         vlc_value_t val_list, text_list;
         var_Change( p_input, "chapter", VLC_VAR_GETCHOICES,
                     &val_list, &text_list );
-        
+
         for( int i = 0; i < val_list.p_list->i_count; i++ )
         {
             message = new BMessage( TOGGLE_CHAPTER );
@@ -1371,6 +1378,7 @@ void ChapterMenu::AttachedToWindow()
         var_Change( p_input, "chapter", VLC_VAR_FREELIST,
                     &val_list, &text_list );
     }
+    vlc_object_release( p_input );
     BMenu::AttachedToWindow();
 }
 
