@@ -1,8 +1,8 @@
 /*****************************************************************************
  * controls.m: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2002-2005 VideoLAN
- * $Id: controls.m 11398 2005-06-10 19:54:49Z hartman $
+ * Copyright (C) 2002-2005 the VideoLAN team
+ * $Id: controls.m 12412 2005-08-27 16:40:23Z jpsaman $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -34,7 +34,7 @@
 #include "vout.h"
 #include "open.h"
 #include "controls.h"
-#include <osd.h>
+#include <vlc_osd.h>
 
 /*****************************************************************************
  * VLCControls implementation 
@@ -250,7 +250,9 @@
 {
     intf_thread_t * p_intf = VLCIntf;
     audio_volume_t i_volume = (audio_volume_t)[sender intValue];
-    aout_VolumeSet( p_intf, i_volume * AOUT_VOLUME_STEP );
+    int i_volume_step = 0;
+    i_volume_step = config_GetInt( p_intf->p_vlc, "volume-step" );
+    aout_VolumeSet( p_intf, i_volume * i_volume_step );
     /* Manage volume status */
     [o_main manageVolumeSlider];
 }
