@@ -1,8 +1,8 @@
 /*****************************************************************************
  * playlist.h: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2002-2004 VideoLAN
- * $Id: playlist.h,v 1.21 2004/02/06 04:51:02 hartman Exp $
+ * Copyright (C) 2002-2005 the VideoLAN team
+ * $Id: playlist.h 12325 2005-08-21 23:33:58Z hartman $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Derk-Jan Hartman <hartman at videolan dot org>
@@ -23,64 +23,114 @@
  *****************************************************************************/
 
 /*****************************************************************************
- * VLCPlaylistView interface 
+ * VLCPlaylistView interface
  *****************************************************************************/
-@interface VLCPlaylistView : NSTableView
+@interface VLCPlaylistView : NSOutlineView
 {
 }
 
 @end
 
 /*****************************************************************************
- * VLCPlaylist interface 
+ * VLCPlaylistCommon interface
  *****************************************************************************/
-@interface VLCPlaylist : NSObject
+@interface VLCPlaylistCommon : NSObject
 {
-    int i_moveRow;
-    bool b_isSortDescending;
-    
-    IBOutlet id o_window;
-    IBOutlet id o_btn_playlist;
-    IBOutlet id o_table_view;
-    
-    IBOutlet id o_status_field;
-    IBOutlet id o_tc_id;
     IBOutlet id o_tc_name;
     IBOutlet id o_tc_author;
     IBOutlet id o_tc_duration;
-    IBOutlet id o_tc_sortColumn;
+    IBOutlet id o_outline_view;
 
-    IBOutlet id o_ctx_menu;
+    NSMutableDictionary *o_outline_dict;
+    int i_current_view;
+}
+
+- (void)initStrings;
+- (playlist_item_t *)selectedPlaylistItem;
+- (NSOutlineView *)outlineView;
+
+@end
+
+/*****************************************************************************
+ * VLCPlaylistWizard interface
+ *****************************************************************************/
+@interface VLCPlaylistWizard : VLCPlaylistCommon
+{
+}
+
+- (IBAction)reloadOutlineView;
+
+@end
+
+/*****************************************************************************
+ * VLCPlaylist interface
+ *****************************************************************************/
+@interface VLCPlaylist : VLCPlaylistCommon
+{
+    IBOutlet id o_controller;
+    IBOutlet id o_playlist_wizard;
+
+    IBOutlet id o_btn_playlist;
+    IBOutlet id o_playlist_view;
+    IBOutlet id o_status_field;
+    IBOutlet id o_search_field;
+    IBOutlet id o_random_ckb;
+    IBOutlet id o_loop_popup;
     IBOutlet id o_mi_save_playlist;
+    IBOutlet id o_ctx_menu;
+
     IBOutlet id o_mi_play;
     IBOutlet id o_mi_delete;
+    IBOutlet id o_mi_info;
+    IBOutlet id o_mi_preparse;
     IBOutlet id o_mi_selectall;
+    IBOutlet id o_mi_sort_name;
+    IBOutlet id o_mi_sort_author;
+    IBOutlet id o_mi_recursive_expand;
 
-    IBOutlet id o_random_ckb;
-    IBOutlet id o_loop_ckb;
-    IBOutlet id o_repeat_ckb;
+    /* "services discovery" menu in the playlist menu */
+    IBOutlet id o_mi_services;
+    IBOutlet id o_mu_services;
 
-    IBOutlet id o_search_keyword;
-    IBOutlet id o_search_button;
+    /* "services discovery" menu in the main menu */
+    IBOutlet id o_mm_mi_services;
+    IBOutlet id o_mm_mu_services;
 
     NSImage *o_descendingSortingImage;
     NSImage *o_ascendingSortingImage;
 
+    NSMutableArray *o_nodes_array;
+    NSMutableArray *o_items_array;
+
+    BOOL b_selected_item_met;
+    BOOL b_isSortDescending;
+    id o_tc_sortColumn;
 }
 
-- (void)initStrings;
+- (void)searchfieldChanged:(NSNotification *)o_notification;
 - (NSMenu *)menuForEvent:(NSEvent *)o_event;
 
-- (IBAction)toggleWindow:(id)sender;
-- (IBAction)savePlaylist:(id)sender;
-- (IBAction)playItem:(id)sender;
-- (IBAction)deleteItems:(id)sender;
-- (IBAction)selectAll:(id)sender;
+- (IBAction)handlePopUp:(id)sender;
 - (IBAction)searchItem:(id)sender;
 
-- (void)appendArray:(NSArray*)o_array atPos:(int)i_position enqueue:(BOOL)b_enqueue;
-
-- (void)updateRowSelection;
 - (void)playlistUpdated;
+- (void)playModeUpdated;
+- (void)sortNode:(int)i_mode;
+- (void)updateRowSelection;
+
+- (IBAction)servicesChange:(id)sender;
+- (IBAction)playItem:(id)sender;
+- (IBAction)preparseItem:(id)sender;
+- (IBAction)savePlaylist:(id)sender;
+- (IBAction)deleteItem:(id)sender;
+- (IBAction)selectAll:(id)sender;
+- (IBAction)sortNodeByName:(id)sender;
+- (IBAction)sortNodeByAuthor:(id)sender;
+- (IBAction)recursiveExpandNode:(id)sender;
+
+- (void)appendArray:(NSArray*)o_array atPos:(int)i_position enqueue:(BOOL)b_enqueue;
+- (void)appendNodeArray:(NSArray*)o_array inNode:(playlist_item_t *)p_node atPos:(int)i_position inView:(int)i_view enqueue:(BOOL)b_enqueue;
+
 
 @end
+
