@@ -1,8 +1,8 @@
 /*****************************************************************************
  r playlistinfo.m: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2002-2004 VideoLAN
- * $Id: playlistinfo.m 8121 2004-07-05 01:27:35Z hartman $
+ * Copyright (C) 2002-2005 the VideoLAN team
+ * $Id: playlistinfo.m 12560 2005-09-15 14:21:38Z hartman $
  *
  * Authors: Benjamin Pracht <bigben at videolan dot org>
  *
@@ -41,18 +41,11 @@
 
     if( self != nil )
     {
-        i_item = -1;
-        o_selected = NULL;
+        p_item = NULL;
     }
     return( self );
 }
 
-- (void)dealloc
-{
-    [o_selected release];
-    [super dealloc];
-}
-
 - (void)awakeFromNib
 {
     [o_info_window setExcludedFromWindowsMenu: TRUE];
@@ -63,9 +56,6 @@
     [o_author_lbl setStringValue: _NS("Author")];
     [o_btn_ok setTitle: _NS("OK")];
     [o_btn_cancel setTitle: _NS("Cancel")];
-    [o_btn_delete_group setTitle: _NS("Delete Group")];
-    [o_btn_add_group setTitle: _NS("Add Group")];
-    [o_group_lbl setStringValue: _NS("Group")];
 }
 
 - (IBAction)togglePlaylistInfoPanel:(id)sender
@@ -76,9 +66,7 @@
     }
     else
     {
-        i_item = [[[VLCMain sharedInstance] getPlaylist] selectedPlaylistItem];
-        o_selected = [[[VLCMain sharedInstance] getPlaylist] selectedPlaylistItemsList];
-        [o_selected retain];
+        p_item = [[[VLCMain sharedInstance] getPlaylist] selectedPlaylistItem];
         [self initPanel:sender];
     }
 }
@@ -95,13 +83,10 @@
         playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
                                           FIND_ANYWHERE );
 
-        if (p_playlist)
+        if( p_playlist )
         {
-            i_item = p_playlist->i_index;
-            o_selected = [NSMutableArray arrayWithObject:
-                            [NSNumber numberWithInt:i_item]];
-            [o_selected retain];
-            vlc_object_release(p_playlist);
+            p_item = p_playlist->status.p_item;
+            vlc_object_release( p_playlist );
         }
         [self initPanel:sender];
     }
@@ -109,46 +94,38 @@
 
 - (void)initPanel:(id)sender
 {
-    intf_thread_t * p_intf = VLCIntf;
-    playlist_t * p_playlist;
+    char *psz_temp;
+    vlc_mutex_lock( &p_item->input.lock );
 
-    p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                          FIND_ANYWHERE );
-
-
-    if( p_playlist )
+    /*fill uri / title / author info */
+    if( p_item->input.psz_uri )
     {
-        char *psz_temp;
-
-        /*fill uri / title / author info */
         [o_uri_txt setStringValue:
-            ([NSString stringWithUTF8String:p_playlist->
-               pp_items[i_item]->input.psz_uri] == nil ) ?
-            [NSString stringWithCString:p_playlist->
-                pp_items[i_item]->input.psz_uri] :
-            [NSString stringWithUTF8String:p_playlist->
-                pp_items[i_item]->input.psz_uri]];
+            ([NSString stringWithUTF8String:p_item->input.psz_uri] == nil ) ?
+            [NSString stringWithCString:p_item->input.psz_uri] :
+            [NSString stringWithUTF8String:p_item->input.psz_uri]];
+    }
 
+    if( p_item->input.psz_name )
+    {
         [o_title_txt setStringValue:
-            ([NSString stringWithUTF8String:p_playlist->
-                pp_items[i_item]->input.psz_name] == nil ) ?
-            [NSString stringWithCString:p_playlist->
-                pp_items[i_item]->input.psz_name] :
-            [NSString stringWithUTF8String:p_playlist->
-                pp_items[i_item]->input.psz_name]];
+            ([NSString stringWithUTF8String:p_item->input.psz_name] == nil ) ?
+            [NSString stringWithCString:p_item->input.psz_name] :
+            [NSString stringWithUTF8String:p_item->input.psz_name]];
+    }
+    vlc_mutex_unlock( &p_item->input.lock );
 
-        psz_temp = playlist_GetInfo( p_playlist, i_item ,_("General"),_("Author") );
+    psz_temp = vlc_input_item_GetInfo( &p_item->input, _("Meta-information"), _("Artist") );
+
+    if( psz_temp )
+    {
         [o_author_txt setStringValue: [NSString stringWithUTF8String: psz_temp]];
         free( psz_temp );
-
-        [[VLCInfoTreeItem rootItem] refresh];
-        [o_outline_view reloadData];
-
-        [self createComboBox];
-        [self handleGroup:self];
-
-        vlc_object_release( p_playlist );
     }
+
+    [[VLCInfoTreeItem rootItem] refresh];
+    [o_outline_view reloadData];
+
     [o_info_window makeKeyAndOrderFront: sender];
 }
 
@@ -160,158 +137,54 @@
 
 - (IBAction)infoOk:(id)sender
 {
-    int i,i_row,c;
     intf_thread_t * p_intf = VLCIntf;
     playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
                                           FIND_ANYWHERE );
     vlc_value_t val;
-    NSNumber * o_number;
 
-
-    if (p_playlist)
+    if( [self isItemInPlaylist: p_item] )
     {
-        vlc_mutex_lock(&p_playlist->pp_items[i_item]->input.lock);
+        vlc_mutex_lock( &p_item->input.lock );
 
-        p_playlist->pp_items[i_item]->input.psz_uri =
-            strdup([[o_uri_txt stringValue] cString]);
-        p_playlist->pp_items[i_item]->input.psz_name =
-            strdup([[o_title_txt stringValue] cString]);
-        playlist_ItemAddInfo(p_playlist->pp_items[i_item],_("General"),_("Author"), [[o_author_txt stringValue] cString]);
-
-        c = (int)[o_selected count];
-
-        if ([[o_group_cbx stringValue] isEqual:
-                    [o_group_cbx objectValueOfSelectedItem]])
-        {
-            for (i = 0 ; i < c ; i++)
-            {
-                o_number = [o_selected lastObject];
-                i_row = [o_number intValue];
-                p_playlist->pp_items[i_row]->i_group = p_playlist->
-                    pp_groups[[o_group_cbx indexOfSelectedItem]]->i_id;
-                [o_selected removeObject: o_number];
-            }
-        }
-        else
-        {
-            playlist_group_t * p_group = playlist_CreateGroup( p_playlist,
-                strdup([[o_group_cbx stringValue] cString]));
-            if (p_group)
-            {
-                for (i = 0 ; i < c ; i++)
-                {
-                    o_number = [o_selected lastObject];
-                    i_row = [o_number intValue];
-                    p_playlist->pp_items[i_row]->i_group = p_group->i_id;
-                    [o_selected removeObject: o_number];
-                }
-            }
-        }
-
-
-        vlc_mutex_unlock(&p_playlist->pp_items[i_item]->input.lock);
+        p_item->input.psz_uri = strdup( [[o_uri_txt stringValue] UTF8String] );
+        p_item->input.psz_name = strdup( [[o_title_txt stringValue] UTF8String] );
+        vlc_mutex_unlock( &p_item->input.lock );
+        vlc_input_item_AddInfo( &p_item->input, _("Meta-information"), _("Artist"), [[o_author_txt stringValue] UTF8String]);
+        
         val.b_bool = VLC_TRUE;
-        var_Set( p_playlist,"intf-change",val );
-        vlc_object_release ( p_playlist );
+        var_Set( p_playlist, "intf-change", val );
     }
+    vlc_object_release( p_playlist );
     [o_info_window orderOut: self];
 }
 
-- (IBAction)handleGroup:(id)sender
+- (playlist_item_t *)getItem
 {
-    intf_thread_t * p_intf = VLCIntf;
-    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                          FIND_ANYWHERE );
-
-    if (p_playlist)
-    {
-        if ([[o_group_cbx stringValue] isEqual:
-                    [o_group_cbx objectValueOfSelectedItem]])
-        {
-            [o_group_color setBackgroundColor:[[[VLCMain sharedInstance] getPlaylist]
-                getColor: p_playlist->pp_groups[
-                [o_group_cbx indexOfSelectedItem]]->i_id]];
-        }
-        else
-        {
-            [o_group_color setBackgroundColor:[[[VLCMain sharedInstance] getPlaylist]
-                getColor:p_playlist->pp_groups[
-                [o_group_cbx numberOfItems] - 1]->i_id + 1]];
-        }
-    vlc_object_release(p_playlist);
-    }
+    return p_item;
 }
 
-- (IBAction)deleteOutlineGroup:(id)sender
-{
-    intf_thread_t * p_intf = VLCIntf;
-    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                          FIND_ANYWHERE );
-
-    if(p_playlist)
-    {
-        if ([[o_group_cbx stringValue] isEqual:
-                    [o_group_cbx objectValueOfSelectedItem]])
-        {
-            [[[VLCMain sharedInstance] getPlaylist] deleteGroup:p_playlist->pp_groups[
-                    [o_group_cbx indexOfSelectedItem]]->i_id];
-            [self createComboBox];
-            [self handleGroup:self];
-            [o_group_cbx reloadData];
-        }
-        else
-        {
-            msg_Warn(p_playlist,"Group doesn't exist, cannot delete");
-        }
-    vlc_object_release(p_playlist);
-    }
-}
-
-- (IBAction)createOutlineGroup:(id)sender;
-{
-    intf_thread_t * p_intf = VLCIntf;
-    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                          FIND_ANYWHERE );
-    if(p_playlist)
-    {
-        playlist_CreateGroup( p_playlist,
-                    strdup([[o_group_cbx stringValue] cString]));
-        [self createComboBox];
-        [o_group_cbx reloadData];
-        [[[VLCMain sharedInstance] getPlaylist] playlistUpdated];
-        vlc_object_release(p_playlist);
-    }
-}
-
--(void)createComboBox
+- (BOOL)isItemInPlaylist:(playlist_item_t *)p_local_item
 {
     intf_thread_t * p_intf = VLCIntf;
     playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
                                           FIND_ANYWHERE );
     int i;
 
-    [o_group_cbx removeAllItems];
-
-    if (p_playlist)
+    if( p_playlist == NULL )
     {
-        for (i = 0; i < p_playlist->i_groups ; i++)
-        {
-            [o_group_cbx addItemWithObjectValue:
-                [NSString stringWithUTF8String:
-                p_playlist->pp_groups[i]->psz_name]];
-            if (p_playlist->pp_items[i_item]->i_group == p_playlist
-                ->pp_groups[i]->i_id)
-            {
-                [o_group_cbx selectItemAtIndex:i];
-            }
-        }
-    vlc_object_release(p_playlist);
+        return NO;
     }
-}
 
-- (int)getItem
-{
-    return i_item;
+    for( i = 0 ; i < p_playlist->i_size ; i++ )
+    {
+        if( p_playlist->pp_items[i] == p_local_item )
+        {
+            vlc_object_release( p_playlist );
+            return YES;
+        }
+    }
+    vlc_object_release( p_playlist );
+    return NO;
 }
 
 @end
@@ -387,7 +260,10 @@ static VLCInfoTreeItem *o_root_item = nil;
         o_value = [o_item_value copy];
         i_object_id = i_id;
         o_parent = o_parent_item;
-        i_item = [[[VLCMain sharedInstance] getInfo] getItem];
+        if( [[VLCMain sharedInstance] getInfo] != nil )
+            p_item = [[[VLCMain sharedInstance] getInfo] getItem];
+        else
+            p_item = NULL;
     }
     return( self );
 }
@@ -399,7 +275,7 @@ static VLCInfoTreeItem *o_root_item = nil;
 
 - (void)dealloc
 {
-    if (o_children != IsALeafNode) [o_children release];
+    if( o_children != IsALeafNode ) [o_children release];
     [o_name release];
     [super dealloc];
 }
@@ -410,56 +286,48 @@ static VLCInfoTreeItem *o_root_item = nil;
 {
     if (o_children == NULL)
     {
-        intf_thread_t * p_intf = VLCIntf;
-        playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                          FIND_ANYWHERE );
         int i;
 
-        if (p_playlist)
+        if( [[[VLCMain sharedInstance] getInfo] isItemInPlaylist: p_item] )
         {
-            if (i_item > -1)
+            if( self == o_root_item )
             {
-                if (self == o_root_item)
+                vlc_mutex_lock( &p_item->input.lock );
+                o_children = [[NSMutableArray alloc] initWithCapacity:
+                                                p_item->input.i_categories];
+                for (i = 0 ; i < p_item->input.i_categories ; i++)
                 {
-                    o_children = [[NSMutableArray alloc] initWithCapacity:p_playlist->pp_items[i_item]->input.i_categories];
-                    for (i = 0 ; i<p_playlist->pp_items[i_item]->input.i_categories ; i++)
-                    {
-                        [o_children addObject:[[VLCInfoTreeItem alloc]
-                            initWithName: [NSString stringWithUTF8String:
-                                p_playlist->pp_items[i_item]->input.
-                                pp_categories[i]->psz_name]
-                            value: @""
-                            ID: i
-                            parent: self]];
-                    }
-                }
-                else if (o_parent == o_root_item)
-                {
-                    o_children = [[NSMutableArray alloc] initWithCapacity:
-                        p_playlist->pp_items[i_item]->input.
-                        pp_categories[i_object_id]->i_infos];
-                    for (i = 0 ; i<p_playlist->pp_items[i_item]->input.
-                           pp_categories[i_object_id]->i_infos ; i++)
-                    {
-                        [o_children addObject:[[VLCInfoTreeItem alloc]
+                    [o_children addObject:[[VLCInfoTreeItem alloc]
                         initWithName: [NSString stringWithUTF8String:
-                                p_playlist->pp_items[i_item]->input.
-                                pp_categories[i_object_id]->
-                                pp_infos[i]->psz_name]
-                            value: [NSString stringWithUTF8String:
-                                p_playlist->pp_items[i_item]->input.
-                                pp_categories[i_object_id]->
-                                pp_infos[i]->psz_value]
-                            ID: i
-                            parent: self]];
-                    }
-                }
-                else
-                {
-                    o_children = IsALeafNode;
+                            p_item->input.pp_categories[i]->psz_name]
+                        value: @""
+                        ID: i
+                        parent: self]];
                 }
+                vlc_mutex_unlock( &p_item->input.lock );
+            }
+            else if( o_parent == o_root_item )
+            {
+                vlc_mutex_lock( &p_item->input.lock );
+                o_children = [[NSMutableArray alloc] initWithCapacity:
+                    p_item->input.pp_categories[i_object_id]->i_infos];
+
+                for (i = 0 ; i < p_item->input.pp_categories[i_object_id]->i_infos ; i++)
+                {
+                    [o_children addObject:[[VLCInfoTreeItem alloc]
+                    initWithName: [NSString stringWithUTF8String:
+                            p_item->input.pp_categories[i_object_id]->pp_infos[i]->psz_name]
+                        value: [NSString stringWithUTF8String:
+                            p_item->input.pp_categories[i_object_id]->pp_infos[i]->psz_value]
+                        ID: i
+                        parent: self]];
+                }
+                vlc_mutex_unlock( &p_item->input.lock );
+            }
+            else
+            {
+                o_children = IsALeafNode;
             }
-            vlc_object_release(p_playlist);
         }
     }
     return o_children;
@@ -481,7 +349,7 @@ static VLCInfoTreeItem *o_root_item = nil;
 
 - (int)numberOfChildren {
     id i_tmp = [self children];
-    return (i_tmp == IsALeafNode) ? (-1) : (int)[i_tmp count];
+    return ( i_tmp == IsALeafNode ) ? (-1) : (int)[i_tmp count];
 }
 
 /*- (int)selectedPlaylistItem
@@ -491,8 +359,8 @@ static VLCInfoTreeItem *o_root_item = nil;
 */
 - (void)refresh
 {
-    i_item = [[[VLCMain sharedInstance] getInfo] getItem];
-    if (o_children != NULL)
+    p_item = [[[VLCMain sharedInstance] getInfo] getItem];
+    if( o_children != NULL )
     {
         [o_children release];
         o_children = NULL;
