@@ -1,8 +1,8 @@
 /*****************************************************************************
  * cmd_vars.hpp
  *****************************************************************************
- * Copyright (C) 2004 VideoLAN
- * $Id: cmd_vars.hpp 9410 2004-11-24 20:23:23Z asmax $
+ * Copyright (C) 2004 the VideoLAN team
+ * $Id: cmd_vars.hpp 12949 2005-10-23 17:42:16Z asmax $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *
@@ -32,6 +32,28 @@ class VarText;
 /// Command to notify the playlist of a change
 DEFINE_COMMAND( NotifyPlaylist, "notify playlist" )
 
+/// Command to notify the playlist of a change
+DEFINE_COMMAND( PlaytreeChanged, "playtree changed" )
+
+/// Command to notify the playtree of an item update
+class CmdPlaytreeUpdate: public CmdGeneric
+{
+    public:
+        CmdPlaytreeUpdate( intf_thread_t *pIntf, int id ):
+            CmdGeneric( pIntf ), m_id( id ) {}
+        virtual ~CmdPlaytreeUpdate() {}
+
+        /// This method does the real job of the command
+        virtual void execute();
+
+        /// Return the type of the command
+        virtual string getType() const { return "playtree update"; }
+
+    private:
+        /// Playlist item ID
+        int m_id;
+};
+
 
 /// Command to set a text variable
 class CmdSetText: public CmdGeneric
