@@ -1,8 +1,8 @@
 /*****************************************************************************
  * window_manager.hpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: window_manager.hpp 7270 2004-04-03 23:21:47Z asmax $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: window_manager.hpp 12912 2005-10-22 11:57:29Z asmax $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulire <ipkiss@via.ecp.fr>
@@ -67,8 +67,11 @@ class WindowManager: public SkinObject
         /// If a new anchoring is detected, the windows will move accordingly.
         void move( TopWindow &rWindow, int left, int top ) const;
 
+        /// Raise all the registered windows
+        void raiseAll() const;
+
         /// Show all the registered windows
-        void showAll() const;
+        void showAll(bool firstTime = false) const;
 
         /// Hide all the registered windows
         void hideAll() const;
@@ -130,7 +133,7 @@ class WindowManager: public SkinObject
         /// move.
         WinSet_t m_movingWindows;
         /// Indicate whether the windows are currently on top
-        bool m_isOnTop;
+        VariablePtr m_cVarOnTop;
         /// Magnetism of the screen edges (= scope of action)
         int m_magnet;
         /// Alpha value of the static windows
