@@ -1,8 +1,8 @@
 /*****************************************************************************
  * x11_factory.cpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: x11_factory.cpp 8116 2004-07-04 22:22:10Z adn $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: x11_factory.cpp 12207 2005-08-15 15:54:32Z asmax $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulire <ipkiss@via.ecp.fr>
@@ -102,9 +102,9 @@ void X11Factory::minimize()
                     DefaultScreen( m_pDisplay->getDisplay() ) );
 }
 
-OSTimer *X11Factory::createOSTimer( const Callback &rCallback )
+OSTimer *X11Factory::createOSTimer( CmdGeneric &rCmd )
 {
-    return new X11Timer( getIntf(), rCallback );
+    return new X11Timer( getIntf(), rCmd );
 }
 
 
