@@ -1,8 +1,8 @@
 /*****************************************************************************
  * x11_window.cpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: x11_window.cpp 8391 2004-08-06 17:28:36Z sam $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: x11_window.cpp 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulire <ipkiss@via.ecp.fr>
@@ -109,10 +109,10 @@ X11Window::X11Window( intf_thread_t *pIntf, GenericWindow &rWindow,
     // Associate the window to the main "parent" window
     XSetTransientForHint( XDISPLAY, m_wnd, m_rDisplay.getMainWindow() );
 
-    // XXX Set this window as the vout
+    // Set this window as a vout
     if( m_pParent )
     {
-        VlcProc::instance( getIntf() )->setVoutWindow( (void*)m_wnd );
+        VlcProc::instance( getIntf() )->registerVoutWindow( (void*)m_wnd );
     }
 
 }
@@ -120,10 +120,9 @@ X11Window::X11Window( intf_thread_t *pIntf, GenericWindow &rWindow,
 
 X11Window::~X11Window()
 {
-    // XXX This window is no more the vout
     if( m_pParent )
     {
-        VlcProc::instance( getIntf() )->setVoutWindow( NULL );
+        VlcProc::instance( getIntf() )->unregisterVoutWindow( (void*)m_wnd );
     }
 
     X11Factory *pFactory = (X11Factory*)X11Factory::instance( getIntf() );
