@@ -1,8 +1,8 @@
 /*****************************************************************************
  * pes.c: PES packetizer used by the MPEG multiplexers
  *****************************************************************************
- * Copyright (C) 2001, 2002 VideoLAN
- * $Id: pes.c 9130 2004-11-03 18:32:30Z gbazin $
+ * Copyright (C) 2001, 2002 the VideoLAN team
+ * $Id: pes.c 12798 2005-10-09 17:01:08Z fenrir $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -44,8 +44,6 @@
 #include "pes.h"
 #include "bits.h"
 
-#define PES_PAYLOAD_SIZE_MAX 65500
-
 static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
                              int i_es_size, es_format_t *p_fmt,
                              int i_stream_id, int i_private_id,
@@ -67,7 +65,6 @@ static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
         }
     }
 
-
     bits_initwrite( &bits, 50, p_hdr );
 
     /* add start code */
@@ -233,7 +230,8 @@ static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
 
 int E_( EStoPES )( sout_instance_t *p_sout, block_t **pp_pes, block_t *p_es,
                    es_format_t *p_fmt, int i_stream_id,
-                   int b_mpeg2, int b_data_alignment, int i_header_size )
+                   int b_mpeg2, int b_data_alignment, int i_header_size,
+                   int i_max_pes_size )
 {
     block_t *p_pes;
     mtime_t i_pts, i_dts, i_length;
@@ -256,6 +254,15 @@ int E_( EStoPES )( sout_instance_t *p_sout, block_t **pp_pes, block_t *p_es,
         i_stream_id  = PES_PRIVATE_STREAM_1;
     }
 
+    if( p_fmt->i_codec == VLC_FOURCC( 'm', 'p','4', 'v' ) &&
+        p_es->i_flags & BLOCK_FLAG_TYPE_I )
+    {
+        /* For MPEG4 video, add VOL before I-frames */
+        p_es = block_Realloc( p_es, p_fmt->i_extra, p_es->i_buffer );
+
+        memcpy( p_es->p_buffer, p_fmt->p_extra, p_fmt->i_extra );
+    }
+
     i_pts = p_es->i_pts <= 0 ? 0 : p_es->i_pts * 9 / 100; // 90000 units clock
     i_dts = p_es->i_dts <= 0 ? 0 : p_es->i_dts * 9 / 100; // 90000 units clock
 
@@ -264,9 +271,14 @@ int E_( EStoPES )( sout_instance_t *p_sout, block_t **pp_pes, block_t *p_es,
 
     *pp_pes = p_pes = NULL;
 
+#ifdef DEBUG
+    memset( header, 0, 50 );
+#endif
+
     do
     {
-        i_pes_payload = __MIN( i_size, PES_PAYLOAD_SIZE_MAX );
+        i_pes_payload = __MIN( i_size, (i_max_pes_size ?
+                               i_max_pes_size : PES_PAYLOAD_SIZE_MAX) );
         i_pes_header  = PESHeader( header, i_pts, i_dts, i_pes_payload,
                                    p_fmt, i_stream_id, i_private_id, b_mpeg2,
                                    b_data_alignment, i_header_size );
