@@ -1,11 +1,12 @@
 /*****************************************************************************
  * transcode.c: transcoding stream output module
  *****************************************************************************
- * Copyright (C) 2003-2004 VideoLAN
- * $Id: transcode.c 11351 2005-06-08 12:07:14Z gbazin $
+ * Copyright (C) 2003-2004 the VideoLAN team
+ * $Id: transcode.c 13101 2005-11-02 18:16:58Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
+ *          Jean-Paul Saman <jpsaman #_at_# m2x dot nl> 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +36,9 @@
 #include <vlc/vout.h>
 #include <vlc/decoder.h>
 #include "vlc_filter.h"
-#include "osd.h"
+#include "vlc_osd.h"
+
+#define MASTER_SYNC_MAX_DRIFT 100000
 
 /*****************************************************************************
  * Module descriptor
@@ -63,14 +66,23 @@
     "Allows you to deinterlace the video before encoding." )
 #define DEINTERLACE_MODULE_TEXT N_("Deinterlace module")
 #define DEINTERLACE_MODULE_LONGTEXT N_( \
-    "Specifies the deinterlace module to use (ffmpeg-deinterlace or " \
-    "deinterlace)." )
+    "Specifies the deinterlace module to use." )
 #define WIDTH_TEXT N_("Video width")
 #define WIDTH_LONGTEXT N_( \
     "Allows you to specify the output video width." )
 #define HEIGHT_TEXT N_("Video height")
 #define HEIGHT_LONGTEXT N_( \
     "Allows you to specify the output video height." )
+#define MAXWIDTH_TEXT N_("Maximum video width")
+#define MAXWIDTH_LONGTEXT N_( \
+    "Allows you to specify a maximum output video width." )
+#define MAXHEIGHT_TEXT N_("Maximum video height")
+#define MAXHEIGHT_LONGTEXT N_( \
+    "Allows you to specify a maximum output video height." )
+#define VFILTER_TEXT N_("Video filter")
+#define VFILTER_LONGTEXT N_( \
+    "Allows you to specify video filters used after the video " \
+    "transcoding and subpictures overlaying." )
 
 #define CROPTOP_TEXT N_("Video crop top")
 #define CROPTOP_LONGTEXT N_( \
@@ -120,9 +132,17 @@
     "transcoding. The subpictures produced by the filters will be overlayed " \
     "directly onto the video." )
 
+#define OSD_TEXT N_("OSD menu")
+#define OSD_LONGTEXT N_(\
+    "Enable streaming of the On Screen Display. It uses the osdmenu subfilter." )
+                  
 #define THREADS_TEXT N_("Number of threads")
 #define THREADS_LONGTEXT N_( \
     "Allows you to specify the number of threads used for the transcoding." )
+#define HP_TEXT N_("High priority")
+#define HP_LONGTEXT N_( \
+    "Runs the optional encoder thread at the OUTPUT priority instead of " \
+    "VIDEO." )
 
 #define ASYNC_TEXT N_("Synchronise on audio track")
 #define ASYNC_LONGTEXT N_( \
@@ -133,6 +153,11 @@
 #define HURRYUP_LONGTEXT N_( "Allows you to specify if the transcoder " \
   "should drop frames if your CPU can't keep up with the encoding rate." )
 
+static char *ppsz_deinterlace_type[] =
+{
+    "deinterlace", "ffmpeg-deinterlace"
+};
+
 static int  Open ( vlc_object_t * );
 static void Close( vlc_object_t * );
 
@@ -164,10 +189,18 @@ vlc_module_begin();
     add_string( SOUT_CFG_PREFIX "deinterlace-module", "deinterlace", NULL,
                 DEINTERLACE_MODULE_TEXT, DEINTERLACE_MODULE_LONGTEXT,
                 VLC_FALSE );
+        change_string_list( ppsz_deinterlace_type, 0, 0 );
     add_integer( SOUT_CFG_PREFIX "width", 0, NULL, WIDTH_TEXT,
                  WIDTH_LONGTEXT, VLC_TRUE );
     add_integer( SOUT_CFG_PREFIX "height", 0, NULL, HEIGHT_TEXT,
                  HEIGHT_LONGTEXT, VLC_TRUE );
+    add_integer( SOUT_CFG_PREFIX "maxwidth", 0, NULL, MAXWIDTH_TEXT,
+                 MAXWIDTH_LONGTEXT, VLC_TRUE );
+    add_integer( SOUT_CFG_PREFIX "maxheight", 0, NULL, MAXHEIGHT_TEXT,
+                 MAXHEIGHT_LONGTEXT, VLC_TRUE );
+    add_module_list_cat( SOUT_CFG_PREFIX "vfilter", SUBCAT_VIDEO_VFILTER,
+                     NULL, NULL,
+                     VFILTER_TEXT, VFILTER_LONGTEXT, VLC_FALSE );
 
     add_integer( SOUT_CFG_PREFIX "croptop", 0, NULL, CROPTOP_TEXT,
                  CROPTOP_LONGTEXT, VLC_TRUE );
@@ -203,18 +236,24 @@ vlc_module_begin();
                      NULL, NULL,
                      SFILTER_TEXT, SFILTER_LONGTEXT, VLC_FALSE );
 
+    set_section( N_("On Screen Display"), NULL );
+    add_bool( SOUT_CFG_PREFIX "osd", 0, NULL, OSD_TEXT,
+              OSD_LONGTEXT, VLC_FALSE );
+    
     set_section( N_("Miscellaneous"), NULL );
     add_integer( SOUT_CFG_PREFIX "threads", 0, NULL, THREADS_TEXT,
                  THREADS_LONGTEXT, VLC_TRUE );
+    add_bool( SOUT_CFG_PREFIX "high-priority", 0, NULL, HP_TEXT, HP_LONGTEXT,
+              VLC_TRUE );
 
 vlc_module_end();
 
 static const char *ppsz_sout_options[] = {
     "venc", "vcodec", "vb", "croptop", "cropbottom", "cropleft", "cropright",
-    "scale", "fps", "width", "height", "deinterlace", "deinterlace-module",
-    "threads", "hurry-up", "aenc", "acodec", "ab", "samplerate", "channels",
-    "senc", "scodec", "soverlay", "sfilter",
-    "audio-sync", NULL
+    "scale", "fps", "width", "height", "vfilter", "deinterlace",
+    "deinterlace-module", "threads", "hurry-up", "aenc", "acodec", "ab",
+    "samplerate", "channels", "senc", "scodec", "soverlay", "sfilter",
+    "osd", "audio-sync", "high-priority", "maxwidth", "maxheight", NULL
 };
 
 /*****************************************************************************
@@ -251,6 +290,11 @@ static void transcode_spu_close  ( sout_stream_t *, sout_stream_id_t * );
 static int  transcode_spu_process( sout_stream_t *, sout_stream_id_t *,
                                    block_t *, block_t ** );
 
+static int  transcode_osd_new    ( sout_stream_t *, sout_stream_id_t * );
+static void transcode_osd_close  ( sout_stream_t *, sout_stream_id_t * );
+static int  transcode_osd_process( sout_stream_t *, sout_stream_id_t *,
+                                   block_t *, block_t ** );
+                                   
 static int  EncoderThread( struct sout_stream_sys_t * p_sys );
 
 static int pi_channels_maps[6] =
@@ -294,13 +338,17 @@ struct sout_stream_sys_t
     int             i_vbitrate;
     double          f_scale;
     double          f_fps;
-    int             i_width;
-    int             i_height;
+    unsigned int    i_width, i_maxwidth;
+    unsigned int    i_height, i_maxheight;
     vlc_bool_t      b_deinterlace;
     char            *psz_deinterlace;
     sout_cfg_t      *p_deinterlace_cfg;
     int             i_threads;
+    vlc_bool_t      b_high_priority;
     vlc_bool_t      b_hurry_up;
+    char            *psz_vfilters[10];
+    sout_cfg_t      *p_vfilters_cfg[10];
+    int             i_vfilters;
 
     int             i_crop_top;
     int             i_crop_bottom;
@@ -314,6 +362,14 @@ struct sout_stream_sys_t
     sout_cfg_t      *p_spu_cfg;
     spu_t           *p_spu;
 
+    /* OSD Menu */
+    sout_stream_id_t *id_osd;   /* extension for streaming OSD menus */
+    vlc_fourcc_t    i_osdcodec; /* codec osd menu (0 if not transcode) */
+    char            *psz_osdenc;
+    sout_cfg_t      *p_osd_cfg;
+    vlc_bool_t      b_es_osd;      /* VLC_TRUE when osd es is registered */
+    vlc_bool_t      b_sout_osd;
+        
     /* Sync */
     vlc_bool_t      b_master_sync;
     mtime_t         i_master_drift;
@@ -389,6 +445,13 @@ static int Open( vlc_object_t *p_this )
 
     if( p_sys->i_acodec )
     {
+        if( (strncmp( (char *)&p_sys->i_acodec, "mp3", 3) == 0) &&
+                            (p_sys->i_channels > 2) )
+        {
+            msg_Warn( p_stream, "%d channels invalid for mp3, forcing to 2",
+                      p_sys->i_channels );
+            p_sys->i_channels = 2;
+        }                    
         msg_Dbg( p_stream, "codec audio=%4.4s %dHz %d channels %dKb/s",
                  (char *)&p_sys->i_acodec, p_sys->i_sample_rate,
                  p_sys->i_channels, p_sys->i_abitrate / 1000 );
@@ -436,6 +499,32 @@ static int Open( vlc_object_t *p_this )
     var_Get( p_stream, SOUT_CFG_PREFIX "height", &val );
     p_sys->i_height = val.i_int;
 
+    var_Get( p_stream, SOUT_CFG_PREFIX "maxwidth", &val );
+    p_sys->i_maxwidth = val.i_int;
+
+    var_Get( p_stream, SOUT_CFG_PREFIX "maxheight", &val );
+    p_sys->i_maxheight = val.i_int;
+
+    var_Get( p_stream, SOUT_CFG_PREFIX "vfilter", &val );
+    p_sys->i_vfilters = 0;
+    if( val.psz_string && *val.psz_string )
+    {
+        char *psz_parser = val.psz_string;
+
+        while( psz_parser != NULL && *psz_parser != '\0' )
+        {
+            psz_parser = sout_CfgCreate(
+                                   &p_sys->psz_vfilters[p_sys->i_vfilters],
+                                   &p_sys->p_vfilters_cfg[p_sys->i_vfilters],
+                                   psz_parser );
+            p_sys->i_vfilters++;
+            if( psz_parser != NULL && *psz_parser != '\0' ) psz_parser++;
+        }
+    }
+    if( val.psz_string ) free( val.psz_string );
+    p_sys->psz_vfilters[p_sys->i_vfilters] = NULL;
+    p_sys->p_vfilters_cfg[p_sys->i_vfilters] = NULL;
+
     var_Get( p_stream, SOUT_CFG_PREFIX "deinterlace", &val );
     p_sys->b_deinterlace = val.b_bool;
 
@@ -463,6 +552,8 @@ static int Open( vlc_object_t *p_this )
 
     var_Get( p_stream, SOUT_CFG_PREFIX "threads", &val );
     p_sys->i_threads = val.i_int;
+    var_Get( p_stream, SOUT_CFG_PREFIX "high-priority", &val );
+    p_sys->b_high_priority = val.b_bool;
 
     if( p_sys->i_vcodec )
     {
@@ -472,7 +563,7 @@ static int Open( vlc_object_t *p_this )
     }
 
     /* Subpictures transcoding parameters */
-    p_sys->p_spu = 0;
+    p_sys->p_spu = NULL;
     p_sys->psz_senc = NULL;
     p_sys->p_spu_cfg = NULL;
     p_sys->i_scodec = 0;
@@ -514,6 +605,45 @@ static int Open( vlc_object_t *p_this )
     }
     if( val.psz_string ) free( val.psz_string );
 
+    /* OSD menu transcoding parameters */
+    p_sys->psz_osdenc = NULL;
+    p_sys->p_osd_cfg  = NULL;
+    p_sys->i_osdcodec = 0;
+    p_sys->b_es_osd   = VLC_FALSE;
+
+    var_Get( p_stream, SOUT_CFG_PREFIX "osd", &val );
+    p_sys->b_sout_osd = val.b_bool;
+    if( p_sys->b_sout_osd )
+    {
+        vlc_value_t osd_val;
+        char *psz_next;
+       
+        psz_next = sout_CfgCreate( &p_sys->psz_osdenc,
+                                   &p_sys->p_osd_cfg, strdup( "dvbsub") );
+        if( psz_next ) free( psz_next );
+        
+        p_sys->i_osdcodec = VLC_FOURCC('Y','U','V','P' );        
+
+        msg_Dbg( p_stream, "codec osd=%4.4s", (char *)&p_sys->i_osdcodec );
+
+        if( !p_sys->p_spu )
+        {
+            osd_val.psz_string = strdup("osdmenu");
+            p_sys->p_spu = spu_Create( p_stream );
+            var_Create( p_sys->p_spu, "sub-filter", VLC_VAR_STRING );
+            var_Set( p_sys->p_spu, "sub-filter", osd_val );
+            spu_Init( p_sys->p_spu );
+            if( osd_val.psz_string ) free( osd_val.psz_string );
+        }
+        else
+        {
+            osd_val.psz_string = strdup("osdmenu");
+            var_Set( p_sys->p_spu, "sub-filter", osd_val );
+            if( osd_val.psz_string ) free( osd_val.psz_string );
+        }         
+    }
+
+    /* Audio settings */
     var_Get( p_stream, SOUT_CFG_PREFIX "audio-sync", &val );
     p_sys->b_master_sync = val.b_bool;
     if( p_sys->f_fps > 0 ) p_sys->b_master_sync = VLC_TRUE;
@@ -593,6 +723,20 @@ static void Close( vlc_object_t * p_this )
     if( p_sys->psz_senc ) free( p_sys->psz_senc );
 
     if( p_sys->p_spu ) spu_Destroy( p_sys->p_spu );
+    
+    while( p_sys->p_osd_cfg != NULL )
+    {
+        sout_cfg_t *p_next = p_sys->p_osd_cfg->p_next;
+
+        if( p_sys->p_osd_cfg->psz_name )
+            free( p_sys->p_osd_cfg->psz_name );
+        if( p_sys->p_osd_cfg->psz_value )
+            free( p_sys->p_osd_cfg->psz_value );
+        free( p_sys->p_osd_cfg );
+
+        p_sys->p_osd_cfg = p_next;
+    }
+    if( p_sys->psz_osdenc ) free( p_sys->psz_osdenc );    
 
     vlc_object_destroy( p_sys );
 }
@@ -610,6 +754,8 @@ struct sout_stream_id_t
     /* Filters */
     filter_t        *pp_filter[10];
     int             i_filter;
+    filter_t        *pp_vfilter[10];
+    int             i_vfilter;
 
     /* Encoder */
     encoder_t       *p_encoder;
@@ -618,7 +764,6 @@ struct sout_stream_id_t
     date_t          interpolated_pts;
 };
 
-
 static sout_stream_id_t *Add( sout_stream_t *p_stream, es_format_t *p_fmt )
 {
     sout_stream_sys_t *p_sys = p_stream->p_sys;
@@ -704,7 +849,11 @@ static sout_stream_id_t *Add( sout_stream_t *p_stream, es_format_t *p_fmt )
         id->id = p_sys->p_out->pf_add( p_sys->p_out, &id->p_encoder->fmt_out );
         id->b_transcode = VLC_TRUE;
 
-        if( !id->id ) goto error;
+        if( !id->id )
+        {
+            transcode_audio_close( p_stream, id );
+            goto error;
+        }
 
         date_Init( &id->interpolated_pts, p_fmt->audio.i_rate, 1 );
     }
@@ -759,7 +908,11 @@ static sout_stream_id_t *Add( sout_stream_t *p_stream, es_format_t *p_fmt )
         id->id = p_sys->p_out->pf_add( p_sys->p_out, &id->p_encoder->fmt_out );
         id->b_transcode = VLC_TRUE;
 
-        if( !id->id ) goto error;
+        if( !id->id )
+        {
+            transcode_spu_close( p_stream, id );
+            goto error;
+        }
     }
     else if( p_fmt->i_cat == SPU_ES && p_sys->b_soverlay )
     {
@@ -785,6 +938,19 @@ static sout_stream_id_t *Add( sout_stream_t *p_stream, es_format_t *p_fmt )
         if( !id->id ) goto error;
     }
 
+    if( p_sys->b_sout_osd )
+    {
+        /* Create a fake OSD menu elementary stream */
+        if( !p_sys->b_es_osd && (p_sys->i_osdcodec != 0 || p_sys->psz_osdenc) )
+        {
+            if( transcode_osd_new( p_stream, p_sys->id_osd ) )
+            {
+                msg_Err( p_stream, "cannot create osd chain" );
+                goto error;
+            }
+            p_sys->b_es_osd = VLC_TRUE;
+        }
+    }
     return id;
 
  error:
@@ -797,6 +963,7 @@ static sout_stream_id_t *Add( sout_stream_t *p_stream, es_format_t *p_fmt )
     if( id->p_encoder )
     {
         vlc_object_detach( id->p_encoder );
+        es_format_Clean( &id->p_encoder->fmt_out );
         vlc_object_destroy( id->p_encoder );
     }
 
@@ -808,6 +975,9 @@ static int Del( sout_stream_t *p_stream, sout_stream_id_t *id )
 {
     sout_stream_sys_t *p_sys = p_stream->p_sys;
 
+    if( p_sys->b_es_osd )
+        transcode_osd_close( p_stream, p_sys->id_osd );
+
     if( id->b_transcode )
     {
         switch( id->p_decoder->fmt_in.i_cat )
@@ -835,6 +1005,7 @@ static int Del( sout_stream_t *p_stream, sout_stream_id_t *id )
     if( id->p_encoder )
     {
         vlc_object_detach( id->p_encoder );
+        es_format_Clean( &id->p_encoder->fmt_out );
         vlc_object_destroy( id->p_encoder );
     }
 
@@ -847,32 +1018,15 @@ static int Send( sout_stream_t *p_stream, sout_stream_id_t *id,
                  block_t *p_buffer )
 {
     sout_stream_sys_t *p_sys = p_stream->p_sys;
-    block_t *p_out;
+    block_t *p_out = NULL;
 
     if( !id->b_transcode && id->id )
     {
-        if( p_sys->b_master_sync && p_sys->i_master_drift )
+        /* Transcode OSD menu pictures. */
+        if( p_sys->b_es_osd )
         {
-            if( p_buffer->i_dts > 0 )
-            {
-                p_buffer->i_dts -= p_sys->i_master_drift;
-                if( p_buffer->i_dts < 0 )
-                {
-                    block_Release( p_buffer );
-                    return VLC_EGENERIC;
-                }
-            }
-            if( p_buffer->i_pts > 0 )
-            {
-                p_buffer->i_pts -= p_sys->i_master_drift;
-                if( p_buffer->i_pts < 0 )
-                {
-                    block_Release( p_buffer );
-                    return VLC_EGENERIC;
-                }
-            }
+            transcode_osd_process( p_stream, id, p_buffer, &p_out );                
         }
-
         return p_sys->p_out->pf_send( p_sys->p_out, id->id, p_buffer );
     }
     else if( !id->b_transcode )
@@ -904,6 +1058,7 @@ static int Send( sout_stream_t *p_stream, sout_stream_id_t *id,
         break;
 
     default:
+        p_out = NULL;
         block_Release( p_buffer );
         break;
     }
@@ -958,7 +1113,7 @@ static filter_t *transcode_audio_filter_new( sout_stream_t *p_stream,
     filter_t *p_filter = vlc_object_create( p_stream, VLC_OBJECT_FILTER );
 
     vlc_object_attach( p_filter, p_stream );
-    p_filter->pf_audio_buffer_new = __block_New;
+    p_filter->pf_audio_buffer_new = (block_t* (*) (filter_t*, int))__block_New;
 
     p_filter->fmt_in = *p_fmt_in;
     p_filter->fmt_out = *p_fmt_out;
@@ -998,7 +1153,7 @@ static int transcode_audio_new( sout_stream_t *p_stream,
     id->p_decoder->pf_decode_audio = 0;
     id->p_decoder->pf_aout_buffer_new = audio_new_buffer;
     id->p_decoder->pf_aout_buffer_del = audio_del_buffer;
-    //id->p_decoder->p_cfg = p_sys->p_video_cfg;
+    /* id->p_decoder->p_cfg = p_sys->p_audio_cfg; */
 
     id->p_decoder->p_module =
         module_Need( id->p_decoder, "decoder", "$codec", 0 );
@@ -1176,6 +1331,13 @@ static int transcode_audio_process( sout_stream_t *p_stream,
         if( p_sys->b_master_sync )
         {
             mtime_t i_dts = date_Get( &id->interpolated_pts ) + 1;
+            if ( p_audio_buf->start_date - i_dts > MASTER_SYNC_MAX_DRIFT
+                  || p_audio_buf->start_date - i_dts < -MASTER_SYNC_MAX_DRIFT )
+            {
+                msg_Dbg( p_stream, "drift is too high, resetting master sync" );
+                date_Set( &id->interpolated_pts, p_audio_buf->start_date );
+                i_dts = p_audio_buf->start_date + 1;
+            }
             p_sys->i_master_drift = p_audio_buf->start_date - i_dts;
             date_Increment( &id->interpolated_pts, p_audio_buf->i_nb_samples );
             p_audio_buf->start_date -= p_sys->i_master_drift;
@@ -1284,7 +1446,7 @@ static int transcode_video_new( sout_stream_t *p_stream, sout_stream_id_t *id )
     for( i = 0; i < PICTURE_RING_SIZE; i++ )
         id->p_decoder->p_owner->pp_pics[i] = 0;
     id->p_decoder->p_owner->p_sys = p_sys;
-    //id->p_decoder->p_cfg = p_sys->p_video_cfg;
+    /* id->p_decoder->p_cfg = p_sys->p_video_cfg; */
 
     id->p_decoder->p_module =
         module_Need( id->p_decoder, "decoder", "$codec", 0 );
@@ -1338,10 +1500,14 @@ static int transcode_video_new( sout_stream_t *p_stream, sout_stream_id_t *id )
     /* Close the encoder.
      * We'll open it only when we have the first frame. */
     module_Unneed( id->p_encoder, id->p_encoder->p_module );
+    if( id->p_encoder->fmt_out.p_extra )
+        free( id->p_encoder->fmt_out.p_extra );
     id->p_encoder->p_module = NULL;
 
     if( p_sys->i_threads >= 1 )
     {
+        int i_priority = p_sys->b_high_priority ? VLC_THREAD_PRIORITY_OUTPUT :
+                           VLC_THREAD_PRIORITY_VIDEO;
         p_sys->id_video = id;
         vlc_mutex_init( p_stream, &p_sys->lock_out );
         vlc_cond_init( p_stream, &p_sys->cond );
@@ -1350,8 +1516,8 @@ static int transcode_video_new( sout_stream_t *p_stream, sout_stream_id_t *id )
         p_sys->i_last_pic = 0;
         p_sys->p_buffers = NULL;
         p_sys->b_die = p_sys->b_error = 0;
-        if( vlc_thread_create( p_sys, "encoder", EncoderThread,
-                               VLC_THREAD_PRIORITY_VIDEO, VLC_FALSE ) )
+        if( vlc_thread_create( p_sys, "encoder", EncoderThread, i_priority,
+                               VLC_FALSE ) )
         {
             msg_Err( p_stream, "cannot spawn encoder thread" );
             module_Unneed( id->p_decoder, id->p_decoder->p_module );
@@ -1360,8 +1526,6 @@ static int transcode_video_new( sout_stream_t *p_stream, sout_stream_id_t *id )
         }
     }
 
-    date_Set( &id->interpolated_pts, 0 );
-
     return VLC_SUCCESS;
 }
 
@@ -1398,6 +1562,13 @@ static int transcode_video_encoder_open( sout_stream_t *p_stream,
             id->p_encoder->fmt_out.video.i_height / (double)i_height * i_width;
     }
 
+    if( p_sys->i_maxwidth
+         && id->p_encoder->fmt_out.video.i_width > p_sys->i_maxwidth )
+        id->p_encoder->fmt_out.video.i_width = p_sys->i_maxwidth;
+    if( p_sys->i_maxheight
+         && id->p_encoder->fmt_out.video.i_height > p_sys->i_maxheight )
+        id->p_encoder->fmt_out.video.i_height = p_sys->i_maxheight;
+
     /* Make sure the size is at least a multiple of 2 */
     id->p_encoder->fmt_out.video.i_width =
         (id->p_encoder->fmt_out.video.i_width + 1) >> 1 << 1;
@@ -1528,6 +1699,23 @@ static void transcode_video_close( sout_stream_t *p_stream,
 
         vlc_object_destroy( id->pp_filter[i] );
     }
+    for( i = 0; i < id->i_vfilter; i++ )
+    {
+        vlc_object_detach( id->pp_vfilter[i] );
+        if( id->pp_vfilter[i]->p_module )
+            module_Unneed( id->pp_vfilter[i], id->pp_vfilter[i]->p_module );
+
+        /* Clean-up pictures ring buffer */
+        for( j = 0; j < PICTURE_RING_SIZE; j++ )
+        {
+            if( id->pp_vfilter[i]->p_owner->pp_pics[j] )
+                video_del_buffer( VLC_OBJECT(id->pp_vfilter[i]),
+                                  id->pp_vfilter[i]->p_owner->pp_pics[j] );
+        }
+        free( id->pp_vfilter[i]->p_owner );
+
+        vlc_object_destroy( id->pp_vfilter[i] );
+    }
 }
 
 static int transcode_video_process( sout_stream_t *p_stream,
@@ -1536,7 +1724,7 @@ static int transcode_video_process( sout_stream_t *p_stream,
 {
     sout_stream_sys_t *p_sys = p_stream->p_sys;
     int i_duplicate = 1, i;
-    picture_t *p_pic;
+    picture_t *p_pic, *p_pic2 = NULL;
     *out = NULL;
 
     while( (p_pic = id->p_decoder->pf_decode_video( id->p_decoder, &in )) )
@@ -1561,13 +1749,14 @@ static int transcode_video_process( sout_stream_t *p_stream,
             mtime_t i_master_drift = p_sys->i_master_drift;
             mtime_t i_pts;
 
-            if( !i_master_drift )
-            {
-                /* No audio track ? */
-                p_sys->i_master_drift = i_master_drift = p_pic->date;
-            }
-
             i_pts = date_Get( &id->interpolated_pts ) + 1;
+            if ( p_pic->date - i_pts > MASTER_SYNC_MAX_DRIFT
+                  || p_pic->date - i_pts < -MASTER_SYNC_MAX_DRIFT )
+            {
+                msg_Dbg( p_stream, "drift is too high, resetting master sync" );
+                date_Set( &id->interpolated_pts, p_pic->date );
+                i_pts = p_pic->date + 1;
+            }
             i_video_drift = p_pic->date - i_pts;
             i_duplicate = 1;
 
@@ -1686,6 +1875,41 @@ static int transcode_video_process( sout_stream_t *p_stream,
                     return VLC_EGENERIC;
                 }
             }
+
+            for( i = 0; i < p_sys->i_vfilters; i++ )
+            {
+                id->pp_vfilter[id->i_vfilter] =
+                    vlc_object_create( p_stream, VLC_OBJECT_FILTER );
+                vlc_object_attach( id->pp_vfilter[id->i_vfilter], p_stream );
+
+                id->pp_vfilter[id->i_vfilter]->pf_vout_buffer_new =
+                    video_new_buffer_filter;
+                id->pp_vfilter[id->i_vfilter]->pf_vout_buffer_del =
+                    video_del_buffer_filter;
+
+                id->pp_vfilter[id->i_vfilter]->fmt_in = id->p_encoder->fmt_in;
+                id->pp_vfilter[id->i_vfilter]->fmt_out = id->p_encoder->fmt_in;
+                id->pp_vfilter[id->i_vfilter]->p_cfg = p_sys->p_vfilters_cfg[i];
+                id->pp_vfilter[id->i_vfilter]->p_module =
+                    module_Need( id->pp_vfilter[id->i_vfilter],
+                          "video filter2", p_sys->psz_vfilters[i], 0 );
+                if( id->pp_vfilter[id->i_vfilter]->p_module )
+                {
+                    id->pp_vfilter[id->i_vfilter]->p_owner =
+                        malloc( sizeof(filter_owner_sys_t) );
+                    for( i = 0; i < PICTURE_RING_SIZE; i++ )
+                        id->pp_vfilter[id->i_vfilter]->p_owner->pp_pics[i] = 0;
+                    id->pp_vfilter[id->i_vfilter]->p_owner->p_sys = p_sys;
+
+                    id->i_vfilter++;
+                }
+                else
+                {
+                    msg_Dbg( p_stream, "no video filter found" );
+                    vlc_object_detach( id->pp_vfilter[id->i_vfilter] );
+                    vlc_object_destroy( id->pp_vfilter[id->i_vfilter] );
+                }
+            }
         }
 
         /* Run filter chain */
@@ -1742,36 +1966,83 @@ static int transcode_video_process( sout_stream_t *p_stream,
                                    i_scale_width, i_scale_height );
         }
 
-        if( p_sys->i_threads >= 1 )
+        /* Run vfilter chain */
+        for( i = 0; i < id->i_vfilter; i++ )
+        {
+            p_pic = id->pp_vfilter[i]->pf_video_filter(id->pp_vfilter[i], p_pic);
+        }
+
+        if( p_sys->i_threads == 0 )
+        {
+            block_t *p_block;
+            p_block = id->p_encoder->pf_encode_video( id->p_encoder, p_pic );
+            block_ChainAppend( out, p_block );
+        }
+
+        if( p_sys->b_master_sync )
+        {
+            mtime_t i_pts = date_Get( &id->interpolated_pts ) + 1;
+            if ( p_pic->date - i_pts > MASTER_SYNC_MAX_DRIFT
+                  || p_pic->date - i_pts < -MASTER_SYNC_MAX_DRIFT )
+            {
+                msg_Dbg( p_stream, "drift is too high, resetting master sync" );
+                date_Set( &id->interpolated_pts, p_pic->date );
+                i_pts = p_pic->date + 1;
+            }
+            date_Increment( &id->interpolated_pts, 1 );
+        }
+
+        if( p_sys->b_master_sync && i_duplicate > 1 )
+        {
+            mtime_t i_pts = date_Get( &id->interpolated_pts ) + 1;
+            if ( p_pic->date - i_pts > MASTER_SYNC_MAX_DRIFT
+                  || p_pic->date - i_pts < -MASTER_SYNC_MAX_DRIFT )
+            {
+                msg_Dbg( p_stream, "drift is too high, resetting master sync" );
+                date_Set( &id->interpolated_pts, p_pic->date );
+                i_pts = p_pic->date + 1;
+            }
+            date_Increment( &id->interpolated_pts, 1 );
+
+            if( p_sys->i_threads >= 1 )
+            {
+                /* We can't modify the picture, we need to duplicate it */
+                p_pic2 = video_new_buffer_decoder( id->p_decoder );
+                if( p_pic2 != NULL )
+                {
+                    vout_CopyPicture( p_stream, p_pic2, p_pic );
+                    p_pic2->date = i_pts;
+                }
+            }
+            else
+            {
+                block_t *p_block;
+                p_pic->date = i_pts;
+                p_block = id->p_encoder->pf_encode_video(id->p_encoder, p_pic);
+                block_ChainAppend( out, p_block );
+            }
+        }
+
+        if( p_sys->i_threads == 0 )
+        {
+            p_pic->pf_release( p_pic );
+        }
+        else
         {
             vlc_mutex_lock( &p_sys->lock_out );
             p_sys->pp_pics[p_sys->i_last_pic++] = p_pic;
             p_sys->i_last_pic %= PICTURE_RING_SIZE;
             *out = p_sys->p_buffers;
             p_sys->p_buffers = NULL;
+            if( p_pic2 != NULL )
+            {
+                p_sys->pp_pics[p_sys->i_last_pic++] = p_pic2;
+                p_sys->i_last_pic %= PICTURE_RING_SIZE;
+            }
             vlc_cond_signal( &p_sys->cond );
             vlc_mutex_unlock( &p_sys->lock_out );
         }
-        else
-        {
-            block_t *p_block;
-            p_block = id->p_encoder->pf_encode_video( id->p_encoder, p_pic );
-            block_ChainAppend( out, p_block );
 
-            if( p_sys->b_master_sync )
-                date_Increment( &id->interpolated_pts, 1 );
-
-            if( p_sys->b_master_sync && i_duplicate > 1 )
-            {
-                mtime_t i_pts = date_Get( &id->interpolated_pts ) + 1;
-                date_Increment( &id->interpolated_pts, 1 );
-                p_pic->date = i_pts;
-                p_block = id->p_encoder->pf_encode_video(id->p_encoder, p_pic);
-                block_ChainAppend( out, p_block );
-            }
-
-            p_pic->pf_release( p_pic );
-        }
     }
 
     return VLC_SUCCESS;
@@ -1806,19 +2077,6 @@ static int EncoderThread( sout_stream_sys_t *p_sys )
         vlc_mutex_lock( &p_sys->lock_out );
         block_ChainAppend( &p_sys->p_buffers, p_block );
 
-        if( p_sys->b_master_sync )
-            date_Increment( &id->interpolated_pts, 1 );
-
-#if 0
-        if( p_sys->b_master_sync && i_duplicate > 1 )
-        {
-            mtime_t i_pts = date_Get( &id->interpolated_pts ) + 1;
-            date_Increment( &id->interpolated_pts, 1 );
-            p_pic->date = i_pts;
-            p_block = id->p_encoder->pf_encode_video(id->p_encoder, p_pic);
-            block_ChainAppend( &p_sys->p_buffers, p_block );
-        }
-#endif
         vlc_mutex_unlock( &p_sys->lock_out );
 
         p_pic->pf_release( p_pic );
@@ -1995,7 +2253,7 @@ static int transcode_spu_new( sout_stream_t *p_stream, sout_stream_id_t *id )
     id->p_decoder->pf_spu_buffer_new = spu_new_buffer;
     id->p_decoder->pf_spu_buffer_del = spu_del_buffer;
     id->p_decoder->p_owner = (decoder_owner_sys_t *)p_stream;
-    //id->p_decoder->p_cfg = p_sys->p_spu_cfg;
+    /* id->p_decoder->p_cfg = p_sys->p_spu_cfg; */
 
     id->p_decoder->p_module =
         module_Need( id->p_decoder, "decoder", "$codec", 0 );
@@ -2098,3 +2356,186 @@ static void spu_del_buffer( decoder_t *p_dec, subpicture_t *p_subpic )
     sout_stream_t *p_stream = (sout_stream_t *)p_dec->p_owner;
     spu_DestroySubpicture( p_stream->p_sys->p_spu, p_subpic );
 }
+
+/*
+ * OSD menu
+ */
+static int transcode_osd_new( sout_stream_t *p_stream, sout_stream_id_t *id )
+{
+    sout_stream_sys_t *p_sys = p_stream->p_sys;
+    es_format_t fmt;
+
+    fmt.i_cat = SPU_ES;
+    fmt.i_id = 0xbd1f; /* pid ?? */
+    fmt.i_group = 3;   /* pmt entry ?? */
+    fmt.i_codec = VLC_FOURCC( 'Y', 'U', 'V', 'A' );
+    fmt.psz_language = strdup( "osd" );
+
+    id = malloc( sizeof( sout_stream_id_t ) );
+    memset( id, 0, sizeof(sout_stream_id_t) );
+
+    id->id = NULL;
+    id->p_decoder = NULL;
+    id->p_encoder = NULL;
+
+    /* Create encoder object */
+    id->p_encoder = vlc_object_create( p_stream, VLC_OBJECT_ENCODER );
+    if( !id->p_encoder )
+    {
+        msg_Err( p_stream, "out of memory" );
+        goto error;
+    }
+    vlc_object_attach( id->p_encoder, p_stream );
+    id->p_encoder->p_module = NULL;
+
+    /* Create fake destination format */
+    es_format_Init( &id->p_encoder->fmt_out, fmt.i_cat, 0 );
+    id->p_encoder->fmt_out.i_id    = fmt.i_id;
+    id->p_encoder->fmt_out.i_group = fmt.i_group;
+    id->p_encoder->fmt_out.psz_language = strdup( fmt.psz_language );
+    
+    if( p_sys->i_osdcodec != 0 || p_sys->psz_osdenc )
+    {
+        msg_Dbg( p_stream, "creating osdmenu transcoding from fcc=`%4.4s' "
+                 "to fcc=`%4.4s'", (char*)&fmt.i_codec,
+                 (char*)&p_sys->i_osdcodec );
+
+        /* Complete destination format */
+        id->p_encoder->fmt_out.i_codec = p_sys->i_osdcodec;
+        
+        /*
+         * Open encoder
+         */
+        
+        /* Initialization of encoder format structures */
+        es_format_Init( &id->p_encoder->fmt_in, fmt.i_cat, fmt.i_codec );
+        id->p_encoder->fmt_in.psz_language = strdup( fmt.psz_language );
+
+        id->p_encoder->p_cfg = p_sys->p_osd_cfg;
+
+        id->p_encoder->p_module =
+            module_Need( id->p_encoder, "encoder", p_sys->psz_osdenc, VLC_TRUE );
+
+        if( !id->p_encoder->p_module )
+        {
+            msg_Err( p_stream, "cannot find encoder" );
+            goto error;
+        }
+        
+        /* open output stream */
+        id->id = p_sys->p_out->pf_add( p_sys->p_out, &id->p_encoder->fmt_out );
+        id->b_transcode = VLC_TRUE;
+
+        if( !id->id ) goto error;
+    }
+    else
+    {
+        msg_Dbg( p_stream, "not transcoding a stream (fcc=`%4.4s')",
+                 (char*)&fmt.i_codec );
+        id->id = p_sys->p_out->pf_add( p_sys->p_out, &fmt );
+        id->b_transcode = VLC_FALSE;
+
+        if( !id->id ) goto error;
+    }
+
+    p_sys->id_osd = id;
+    p_sys->b_es_osd = VLC_TRUE;
+
+    if( !p_sys->p_spu )
+    {
+        p_sys->p_spu = spu_Create( p_stream );
+        if( spu_Init( p_sys->p_spu ) != VLC_SUCCESS )
+            msg_Err( p_sys, "spu initialisation failed" );
+    }
+    
+    if( fmt.psz_language )
+        free( fmt.psz_language );
+        
+    return VLC_SUCCESS;
+    
+ error:
+    msg_Err( p_stream, "starting osd encoding thread failed" );
+    if( id->p_encoder->p_module )
+            module_Unneed( id->p_encoder, id->p_encoder->p_module );
+    if( id->p_encoder )
+    {
+        vlc_object_detach( id->p_encoder );
+        vlc_object_destroy( id->p_encoder );
+    }
+    if( fmt.psz_language ) free( fmt.psz_language );
+    if( id ) free( id );
+    p_sys->id_osd = NULL;
+    p_sys->b_es_osd = VLC_FALSE;
+    return VLC_EGENERIC;
+}
+    
+static void transcode_osd_close( sout_stream_t *p_stream, sout_stream_id_t *id)
+{    
+    sout_stream_sys_t *p_sys = p_stream->p_sys;
+    
+    /* Close encoder */
+    if( p_sys->b_es_osd && id )
+    {
+        if( id->p_encoder->p_module )
+            module_Unneed( id->p_encoder, id->p_encoder->p_module );
+        
+        if( id->id ) p_sys->p_out->pf_del( p_sys->p_out, id->id );
+    
+        if( id->p_encoder )
+        {
+            vlc_object_detach( id->p_encoder );
+            vlc_object_destroy( id->p_encoder );
+        }
+    }
+    p_sys->b_es_osd = VLC_FALSE;
+    if( id ) free( id );
+}
+
+static int transcode_osd_process( sout_stream_t *p_stream,
+                                  sout_stream_id_t *id,
+                                  block_t *in, block_t **out )
+{
+    sout_stream_sys_t *p_sys = p_stream->p_sys;
+    subpicture_t *p_subpic = NULL;
+    
+    /* Check if we have a subpicture to send */
+    if( p_sys->p_spu && in->i_dts > 0)
+    {
+        p_subpic = spu_SortSubpictures( p_sys->p_spu, in->i_dts );
+    }
+    else
+    {
+        msg_Warn( p_stream, "spu channel not initialized, doing it now" );
+        if( !p_sys->p_spu )
+        {
+            p_sys->p_spu = spu_Create( p_stream );
+            if( spu_Init( p_sys->p_spu ) != VLC_SUCCESS )
+                msg_Err( p_stream, "spu initialisation failed" );
+        }
+    }
+        
+    if( p_subpic )
+    {
+        block_t *p_block = NULL;
+        
+        if( p_sys->b_master_sync && p_sys->i_master_drift )
+        {
+            p_subpic->i_start -= p_sys->i_master_drift;
+            if( p_subpic->i_stop ) p_subpic->i_stop -= p_sys->i_master_drift;
+        }
+    
+        p_block = p_sys->id_osd->p_encoder->pf_encode_sub( p_sys->id_osd->p_encoder, p_subpic );
+        if( p_block )
+        {
+            p_block->i_dts = p_block->i_pts = in->i_dts;
+            block_ChainAppend( out, p_block );
+            if( *out )
+            {
+                if( p_sys->p_out->pf_send( p_sys->p_out, p_sys->id_osd->id, *out ) == VLC_SUCCESS )
+                    spu_DestroySubpicture( p_sys->p_spu, p_subpic );
+            }            
+            return VLC_SUCCESS;            
+        }
+    }
+    return VLC_EGENERIC;
+}
