@@ -1,8 +1,8 @@
 /*****************************************************************************
  * motiondetect.c : Motion detect video effect plugin for vlc
  *****************************************************************************
- * Copyright (C) 2005 VideoLAN
- * $Id: $
+ * Copyright (C) 2005 the VideoLAN team
+ * $Id: motiondetect.c 12968 2005-10-25 19:24:21Z gbazin $
  *
  * Authors: Jrme Decoodt <djc@videolan.org>
  *
@@ -59,9 +59,8 @@ static int  SendEvents   ( vlc_object_t *, char const *,
 vlc_module_begin();
     set_description( _("Motion detect video filter") );
     set_shortname( N_( "Motion detect" ));
-/*  Leave this commented as long as the VLM intf in wx isn't available
     set_category( CAT_VIDEO );
-    set_subcategory( SUBCAT_VIDEO_VFILTER );*/
+    set_subcategory( SUBCAT_VIDEO_VFILTER );
     set_capability( "video filter", 0 );
 
     add_integer( "motiondetect-history", 1, NULL, HISTORY_TEXT,
@@ -223,14 +222,8 @@ static int Init( vout_thread_t *p_vout )
     p_vout->output.i_width  = p_vout->render.i_width;
     p_vout->output.i_height = p_vout->render.i_height;
     p_vout->output.i_aspect = p_vout->render.i_aspect;
-
-    fmt.i_width = fmt.i_visible_width = p_vout->render.i_width;
-    fmt.i_height = fmt.i_visible_height = p_vout->render.i_height;
-    fmt.i_x_offset = fmt.i_y_offset = 0;
-    fmt.i_chroma = p_vout->render.i_chroma;
-    fmt.i_aspect = p_vout->render.i_aspect;
-    fmt.i_sar_num = p_vout->render.i_aspect * fmt.i_height / fmt.i_width;
-    fmt.i_sar_den = VOUT_ASPECT_FACTOR;
+    p_vout->fmt_out = p_vout->fmt_in;
+    fmt = p_vout->fmt_out;
 
     /* Try to open the real video output */
     msg_Dbg( p_vout, "spawning the real video output" );
