@@ -1,8 +1,8 @@
 /*****************************************************************************
  * transform.c : transform image module for vlc
  *****************************************************************************
- * Copyright (C) 2000-2004 VideoLAN
- * $Id: transform.c 8551 2004-08-28 17:36:02Z gbazin $
+ * Copyright (C) 2000-2004 the VideoLAN team
+ * $Id: transform.c 12968 2005-10-25 19:24:21Z gbazin $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -64,7 +64,10 @@ static char *type_list_text[] = { N_("Rotate by 90 degrees"),
 
 vlc_module_begin();
     set_description( _("Video transformation filter") );
+    set_shortname( N_("Transformation"));
     set_capability( "video filter", 0 );
+    set_category( CAT_VIDEO );
+    set_subcategory( SUBCAT_VIDEO_VFILTER );
 
     add_string( "transform-type", "90", NULL,
                           TYPE_TEXT, TYPE_LONGTEXT, VLC_FALSE);
@@ -177,6 +180,7 @@ static int Init( vout_thread_t *p_vout )
 {
     int i_index;
     picture_t *p_pic;
+    video_format_t fmt = {0};
 
     I_OUTPUTPICTURES = 0;
 
@@ -185,24 +189,33 @@ static int Init( vout_thread_t *p_vout )
     p_vout->output.i_width  = p_vout->render.i_width;
     p_vout->output.i_height = p_vout->render.i_height;
     p_vout->output.i_aspect = p_vout->render.i_aspect;
+    p_vout->fmt_out = p_vout->fmt_in;
+    fmt = p_vout->fmt_out;
 
     /* Try to open the real video output */
     msg_Dbg( p_vout, "spawning the real video output" );
 
     if( p_vout->p_sys->b_rotation )
     {
-        p_vout->p_sys->p_vout = vout_Create( p_vout,
-                           p_vout->render.i_height, p_vout->render.i_width,
-                           p_vout->render.i_chroma,
-                           (uint64_t)VOUT_ASPECT_FACTOR
-                            * (uint64_t)VOUT_ASPECT_FACTOR
-                            / (uint64_t)p_vout->render.i_aspect );
+        fmt.i_width = p_vout->fmt_out.i_height;
+        fmt.i_visible_width = p_vout->fmt_out.i_visible_height;
+        fmt.i_x_offset = p_vout->fmt_out.i_y_offset;
+
+        fmt.i_height = p_vout->fmt_out.i_width;
+        fmt.i_visible_height = p_vout->fmt_out.i_visible_width;
+        fmt.i_y_offset = p_vout->fmt_out.i_x_offset;
+
+        fmt.i_aspect = VOUT_ASPECT_FACTOR *
+            (uint64_t)VOUT_ASPECT_FACTOR / fmt.i_aspect;
+
+        fmt.i_sar_num = p_vout->fmt_out.i_sar_den;
+        fmt.i_sar_den = p_vout->fmt_out.i_sar_num;
+
+        p_vout->p_sys->p_vout = vout_Create( p_vout, &fmt );
     }
     else
     {
-        p_vout->p_sys->p_vout = vout_Create( p_vout,
-                           p_vout->render.i_width, p_vout->render.i_height,
-                           p_vout->render.i_chroma, p_vout->render.i_aspect );
+        p_vout->p_sys->p_vout = vout_Create( p_vout, &fmt );
     }
 
     /* Everything failed */
