@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vlcshell.cpp: a VLC plugin for Mozilla
  *****************************************************************************
- * Copyright (C) 2002-2005 VideoLAN
- * $Id: vlcshell.cpp 11478 2005-06-20 19:44:46Z zorglub $
+ * Copyright (C) 2002-2005 the VideoLAN team
+ * $Id: vlcshell.cpp 13236 2005-11-14 00:52:36Z adn $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -90,10 +90,10 @@
 ******************************************************************************/
 #ifdef XP_UNIX
 #   define VOUT_PLUGINS "xvideo,x11,dummy"
-#   define AOUT_PLUGINS "oss,dummy"
+#   define AOUT_PLUGINS "alsa,oss,dummy"
 
-static int i_previous_height = -1;
-static int i_previous_width = -1;
+static unsigned int i_previous_height = 100000;
+static unsigned int i_previous_width = 100000;
 
 static void Redraw( Widget w, XtPointer closure, XEvent *event );
 static void Resize( Widget w, XtPointer closure, XEvent *event );
@@ -947,6 +947,8 @@ static void Resize ( Widget w, XtPointer closure, XEvent *event )
                  event->xconfigure.height,
                  event->xconfigure.send_event ? "TRUE" : "FALSE" );
     }
+#endif /* X11_RESIZE_DEBUG */
+
     if( p_plugin->i_height == i_previous_height &&
         p_plugin->i_width == i_previous_width )
     {
@@ -955,7 +957,6 @@ static void Resize ( Widget w, XtPointer closure, XEvent *event )
     i_previous_height = p_plugin->i_height;
     i_previous_width  = p_plugin->i_width;
 
-#endif /* X11_RESIZE_DEBUG */
 
     i_ret = XResizeWindow( p_plugin->p_display, p_plugin->window,
             p_plugin->i_width, p_plugin->i_height );
