@@ -1,8 +1,8 @@
 /*****************************************************************************
  * common.c : audio output management of common data structures
  *****************************************************************************
- * Copyright (C) 2002-2004 VideoLAN
- * $Id: common.c 8860 2004-09-30 16:46:37Z gbazin $
+ * Copyright (C) 2002-2005 the VideoLAN team
+ * $Id: common.c 13671 2005-12-10 19:26:03Z hartman $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -129,12 +129,20 @@ void aout_FormatPrepare( audio_sample_format_t * p_format )
         i_result = 2;
         break;
 
+    case VLC_FOURCC('u','2','4','l'):
+    case VLC_FOURCC('s','2','4','l'):
+    case VLC_FOURCC('u','2','4','b'):
+    case VLC_FOURCC('s','2','4','b'):
+        i_result = 3;
+        break;
+
     case VLC_FOURCC('f','l','3','2'):
     case VLC_FOURCC('f','i','3','2'):
         i_result = 4;
         break;
 
     case VLC_FOURCC('s','p','d','i'):
+    case VLC_FOURCC('s','p','d','b'): /* Big endian spdif output */
     case VLC_FOURCC('a','5','2',' '):
     case VLC_FOURCC('d','t','s',' '):
     case VLC_FOURCC('m','p','g','a'):
