@@ -3,8 +3,8 @@
  * This library provides an interface to the message queue to be used by other
  * modules, especially intf modules. See config.h for output configuration.
  *****************************************************************************
- * Copyright (C) 1998-2002 VideoLAN
- * $Id: messages.c,v 1.37 2003/12/04 17:15:59 gbazin Exp $
+ * Copyright (C) 1998-2005 the VideoLAN team
+ * $Id: messages.c 12729 2005-10-02 08:00:06Z courmisch $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -38,9 +38,7 @@
 #   include <fcntl.h>                  /* O_CREAT, O_TRUNC, O_WRONLY, O_SYNC */
 #endif
 
-#ifdef HAVE_ERRNO_H
-#   include <errno.h>                                               /* errno */
-#endif
+#include <errno.h>                                                  /* errno */
 
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>                                   /* close(), write() */
@@ -101,18 +99,19 @@ void __msg_Create( vlc_object_t *p_this )
  */
 void __msg_Flush( vlc_object_t *p_this )
 {
-    int i_index;
-
     vlc_mutex_lock( &p_this->p_libvlc->msg_bank.lock );
 
     p_this->p_libvlc->msg_bank.b_configured = VLC_TRUE;
 
+#if 0
+    /* Some messages remain in the queue, dont rewrite them */
     for( i_index = p_this->p_libvlc->msg_bank.i_start;
          i_index != p_this->p_libvlc->msg_bank.i_stop;
          i_index = (i_index+1) % VLC_MSG_QSIZE )
     {
         PrintMsg( p_this, &p_this->p_libvlc->msg_bank.msg[i_index] );
     }
+#endif
 
     FlushMsg( &p_this->p_libvlc->msg_bank );
 
@@ -211,6 +210,7 @@ void __msg_Unsubscribe( vlc_object_t *p_this, msg_subscription_t *p_sub )
     /* Remove this subscription */
     REMOVE_ELEM( p_bank->pp_sub, p_bank->i_sub, i_index );
 
+    if( p_sub ) free( p_sub );
     vlc_mutex_unlock( &p_bank->lock );
 }
 
@@ -299,12 +299,8 @@ static void QueueMsg( vlc_object_t *p_this, int i_type, const char *psz_module,
 
     if( psz_str == NULL )
     {
-#ifdef HAVE_ERRNO_H
         fprintf( stderr, "main warning: can't store message (%s): ",
                  strerror(errno) );
-#else
-        fprintf( stderr, "main warning: can't store message: " );
-#endif
         vlc_va_copy( args, _args );
         vfprintf( stderr, psz_format, args );
         va_end( args );
@@ -381,8 +377,10 @@ static void QueueMsg( vlc_object_t *p_this, int i_type, const char *psz_module,
 
     if( p_bank->b_overflow )
     {
-        free( p_item->psz_module );
-        free( p_item->psz_msg );
+        if( p_item->psz_module )
+            free( p_item->psz_module );
+        if( p_item->psz_msg )
+            free( p_item->psz_msg );
     }
 
     vlc_mutex_unlock( &p_bank->lock );
@@ -430,8 +428,10 @@ static void FlushMsg ( msg_bank_t *p_bank )
          i_index != i_stop;
          i_index = (i_index+1) % VLC_MSG_QSIZE )
     {
-        free( p_bank->msg[i_index].psz_msg );
-        free( p_bank->msg[i_index].psz_module );
+        if( p_bank->msg[i_index].psz_msg )
+            free( p_bank->msg[i_index].psz_msg );
+        if( p_bank->msg[i_index].psz_module )
+            free( p_bank->msg[i_index].psz_module );
     }
 
     /* Update the new start value */
@@ -491,6 +491,12 @@ static void PrintMsg ( vlc_object_t * p_this, msg_item_t * p_item )
         case VLC_OBJECT_VOUT: psz_object = "video output"; break;
         case VLC_OBJECT_AOUT: psz_object = "audio output"; break;
         case VLC_OBJECT_SOUT: psz_object = "stream output"; break;
+        case VLC_OBJECT_HTTPD: psz_object = "http daemon"; break;
+        case VLC_OBJECT_DIALOGS: psz_object = "dialogs provider"; break;
+        case VLC_OBJECT_VLM: psz_object = "vlm"; break;
+        case VLC_OBJECT_ANNOUNCE: psz_object = "announce handler"; break;
+        case VLC_OBJECT_DEMUX: psz_object = "demuxer"; break;
+        case VLC_OBJECT_ACCESS: psz_object = "access"; break;
     }
 
 #ifdef UNDER_CE
