@@ -1,8 +1,8 @@
 /*****************************************************************************
  * objects.c: vlc_object_t handling
  *****************************************************************************
- * Copyright (C) 2004 VideoLAN
- * $Id: objects.c 8709 2004-09-15 15:50:54Z gbazin $
+ * Copyright (C) 2004 the VideoLAN team
+ * $Id: objects.c 12116 2005-08-10 22:08:50Z jpsaman $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -53,6 +53,9 @@
 #include "vlc_httpd.h"
 #include "vlc_vlm.h"
 #include "vlc_vod.h"
+#include "vlc_tls.h"
+#include "vlc_xml.h"
+#include "vlc_osd.h"
 
 /*****************************************************************************
  * Local prototypes
@@ -69,7 +72,7 @@ static void           SetAttachment ( vlc_object_t *, vlc_bool_t );
 
 static vlc_list_t   * NewList       ( int );
 static void           ListReplace   ( vlc_list_t *, vlc_object_t *, int );
-static void           ListAppend    ( vlc_list_t *, vlc_object_t * );
+/*static void           ListAppend    ( vlc_list_t *, vlc_object_t * );*/
 static int            CountChildren ( vlc_object_t *, int );
 static void           ListChildren  ( vlc_list_t *, vlc_object_t *, int );
 
@@ -125,6 +128,10 @@ void * __vlc_object_create( vlc_object_t *p_this, int i_type )
             i_size = sizeof(playlist_t);
             psz_type = "playlist";
             break;
+        case VLC_OBJECT_SD:
+            i_size = sizeof(services_discovery_t);
+            psz_type = "services discovery";
+            break;
         case VLC_OBJECT_INPUT:
             i_size = sizeof(input_thread_t);
             psz_type = "input";
@@ -185,6 +192,14 @@ void * __vlc_object_create( vlc_object_t *p_this, int i_type )
             i_size = sizeof( vod_t );
             psz_type = "vod server";
             break;
+        case VLC_OBJECT_TLS:
+            i_size = sizeof( tls_t );
+            psz_type = "tls";
+            break;
+        case VLC_OBJECT_XML:
+            i_size = sizeof( xml_t );
+            psz_type = "xml";
+            break;
         case VLC_OBJECT_OPENGL:
             i_size = sizeof( vout_thread_t );
             psz_type = "opengl provider";
@@ -193,6 +208,10 @@ void * __vlc_object_create( vlc_object_t *p_this, int i_type )
             i_size = sizeof( announce_handler_t );
             psz_type = "announce handler";
             break;
+        case VLC_OBJECT_OSDMENU:
+            i_size = sizeof( osd_menu_t );
+            psz_type = "osd menu";
+            break;
         default:
             i_size = i_type > 0
                       ? i_type > (int)sizeof(vlc_object_t)
@@ -230,7 +249,8 @@ void * __vlc_object_create( vlc_object_t *p_this, int i_type )
 
     if( !p_new->p_vars )
     {
-        free( p_new );
+        if( i_type != VLC_OBJECT_ROOT )
+            free( p_new );
         return NULL;
     }
 
@@ -327,17 +347,23 @@ void __vlc_object_destroy( vlc_object_t *p_this )
         /* Don't warn immediately ... 100ms seems OK */
         if( i_delay == 2 )
         {
-            msg_Warn( p_this, "refcount is %i, delaying before deletion",
-                              p_this->i_refcount );
+            msg_Warn( p_this,
+                  "refcount is %i, delaying before deletion (id=%d,type=%d)",
+                  p_this->i_refcount, p_this->i_object_id,
+                  p_this->i_object_type );
         }
-        else if( i_delay == 12 )
+        else if( i_delay == 10 )
         {
-            msg_Err( p_this, "refcount is %i, I have a bad feeling about this",
-                             p_this->i_refcount );
+            msg_Err( p_this,
+                  "refcount is %i, delaying again (id=%d,type=%d)",
+                  p_this->i_refcount, p_this->i_object_id,
+                  p_this->i_object_type );
         }
-        else if( i_delay == 42 )
+        else if( i_delay == 20 )
         {
-            msg_Err( p_this, "we waited too long, cancelling destruction" );
+            msg_Err( p_this,
+                  "we waited too long, cancelling destruction (id=%d,type=%d)",
+                  p_this->i_object_id, p_this->i_object_type );
             return;
         }
 
@@ -382,7 +408,9 @@ void __vlc_object_destroy( vlc_object_t *p_this )
     vlc_mutex_destroy( &p_this->object_lock );
     vlc_cond_destroy( &p_this->object_wait );
 
-    free( p_this );
+    /* root is not dynamically allocated by vlc_object_create */
+    if( p_this->i_object_type != VLC_OBJECT_ROOT )
+        free( p_this );
 }
 
 /**
@@ -1012,7 +1040,7 @@ static void ListReplace( vlc_list_t *p_list, vlc_object_t *p_object,
     return;
 }
 
-static void ListAppend( vlc_list_t *p_list, vlc_object_t *p_object )
+/*static void ListAppend( vlc_list_t *p_list, vlc_object_t *p_object )
 {
     if( p_list == NULL )
     {
@@ -1033,7 +1061,7 @@ static void ListAppend( vlc_list_t *p_list, vlc_object_t *p_object )
     p_list->i_count++;
 
     return;
-}
+}*/
 
 static int CountChildren( vlc_object_t *p_this, int i_type )
 {
