@@ -1,8 +1,8 @@
 /*****************************************************************************
  * win32_specific.c: Win32 specific features
  *****************************************************************************
- * Copyright (C) 2001-2004 VideoLAN
- * $Id: win32_specific.c 9106 2004-11-02 12:52:49Z gbazin $
+ * Copyright (C) 2001-2004 the VideoLAN team
+ * $Id: win32_specific.c 12816 2005-10-11 09:01:00Z damienf $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -37,7 +37,7 @@
 #   include <fcntl.h>
 #endif
 
-#include <winsock2.h>
+#include <winsock.h>
 
 /*****************************************************************************
  * system_Init: initialize winsock and misc other things.
@@ -77,8 +77,8 @@ void system_Init( vlc_t *p_this, int *pi_argc, char *ppsz_argv[] )
     /* Set the default file-translation mode */
 #if !defined( UNDER_CE )
     _fmode = _O_BINARY;
-#endif
     _setmode( _fileno( stdin ), _O_BINARY ); /* Needed for pipes */
+#endif
 
     /* Call mdate() once to make sure it is initialized properly */
     mdate();
@@ -156,7 +156,7 @@ void system_Configure( vlc_t *p_this, int *pi_argc, char *ppsz_argv[] )
         msg_Info( p_this, "one instance mode ENABLED");
 
         /* Use a named mutex to check if another instance is already running */
-        if( ( hmutex = CreateMutex( NULL, TRUE, "VLC ipc "VERSION ) ) == NULL )
+        if( !( hmutex = CreateMutex( 0, TRUE, _T("VLC ipc ") _T(VERSION) ) ) )
         {
             /* Failed for some reason. Just ignore the option and go on as
              * normal. */
@@ -195,8 +195,7 @@ void system_Configure( vlc_t *p_this, int *pi_argc, char *ppsz_argv[] )
 
             /* Locate the window created by the IPC helper thread of the
              * 1st instance */
-            if( ( ipcwindow = FindWindow( NULL, "VLC ipc "VERSION ) )
-                == NULL )
+            if( !( ipcwindow = FindWindow( 0, _T("VLC ipc ") _T(VERSION) ) ) )
             {
                 msg_Err( p_this, "one instance mode DISABLED "
                          "(couldn't find 1st instance of program)" );
@@ -257,8 +256,8 @@ static void IPCHelperThread( vlc_object_t *p_this )
     MSG message;
 
     ipcwindow =
-        CreateWindow( "STATIC",                      /* name of window class */
-                  "VLC ipc "VERSION,                /* window title bar text */
+        CreateWindow( _T("STATIC"),                  /* name of window class */
+                  _T("VLC ipc ") _T(VERSION),       /* window title bar text */
                   0,                                         /* window style */
                   0,                                 /* default X coordinate */
                   0,                                 /* default Y coordinate */
@@ -327,12 +326,20 @@ LRESULT CALLBACK WMCOPYWNDPROC( HWND hwnd, UINT uMsg, WPARAM wParam,
                 {
                     i_options++;
                 }
-
-                playlist_AddExt( p_playlist, ppsz_argv[i_opt],ppsz_argv[i_opt],
-                    PLAYLIST_APPEND | (i_opt? 0 : PLAYLIST_GO),
+                if( i_opt || config_GetInt( p_this, "playlist-enqueue" ) )
+                {
+                  playlist_AddExt( p_playlist, ppsz_argv[i_opt],
+                    ppsz_argv[i_opt], PLAYLIST_APPEND ,
                     PLAYLIST_END, -1,
                     (char const **)( i_options ? &ppsz_argv[i_opt+1] : NULL ),
                     i_options );
+                } else {
+                  playlist_AddExt( p_playlist, ppsz_argv[i_opt],
+                    ppsz_argv[i_opt], PLAYLIST_APPEND | PLAYLIST_GO,
+                    PLAYLIST_END, -1,
+                    (char const **)( i_options ? &ppsz_argv[i_opt+1] : NULL ),
+                    i_options );
+                }
 
                 i_opt += i_options;
             }
@@ -351,5 +358,11 @@ LRESULT CALLBACK WMCOPYWNDPROC( HWND hwnd, UINT uMsg, WPARAM wParam,
  *****************************************************************************/
 void system_End( vlc_t *p_this )
 {
+    if( p_this && p_this->p_libvlc && p_this->p_libvlc->psz_vlcpath )
+    {
+        free( p_this->p_libvlc->psz_vlcpath );
+        p_this->p_libvlc->psz_vlcpath = NULL;
+    }
+
     WSACleanup();
 }
