@@ -1,12 +1,13 @@
 /*****************************************************************************
  * vlc.c: the vlc player
  *****************************************************************************
- * Copyright (C) 1998-2004 VideoLAN
- * $Id: vlc.c,v 1.21 2004/01/25 17:16:05 zorglub Exp $
+ * Copyright (C) 1998-2004 the VideoLAN team
+ * $Id: vlc.c 11664 2005-07-09 06:17:09Z courmisch $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
- *          Gildas Bazin <gbazin@netcourrier.com>
+ *          Gildas Bazin <gbazin@videolan.org>
+ *          Derk-Jan Hartman <hartman at videolan dot org>
  *          Lots of other people, see the libvlc AUTHORS file
  *
  * This program is free software; you can redistribute it and/or modify
@@ -40,18 +41,21 @@
 /*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
-#ifndef WIN32
+#if !defined(WIN32) && !defined(UNDER_CE)
 static void SigHandler  ( int i_signal );
 #endif
 
 /*****************************************************************************
- * main: parse command line, start interface and spawn threads
+ * main: parse command line, start interface and spawn threads.
  *****************************************************************************/
 int main( int i_argc, char *ppsz_argv[] )
 {
     int i_ret;
 
+#ifndef SYS_DARWIN
+    /* This clutters OSX GUI error logs */
     fprintf( stderr, "VLC media player %s\n", VLC_Version() );
+#endif
 
 #ifdef HAVE_PUTENV
 #   ifdef DEBUG
@@ -76,7 +80,7 @@ int main( int i_argc, char *ppsz_argv[] )
         return i_ret;
     }
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(UNDER_CE)
     /* Set the signal handlers. SIGTERM is not intercepted, because we need at
      * least one method to kill the program when all other methods failed, and
      * when we don't want to use SIGKILL.
@@ -98,14 +102,10 @@ int main( int i_argc, char *ppsz_argv[] )
         return i_ret;
     }
 
-    /* Run libvlc, in non-blocking mode */
-    i_ret = VLC_Play( 0 );
-
-    /* Add a blocking interface and keep the return value */
-    i_ret = VLC_AddIntf( 0, NULL, VLC_TRUE );
+    i_ret = VLC_AddIntf( 0, NULL, VLC_TRUE, VLC_TRUE );
 
     /* Finish the threads */
-    VLC_Stop( 0 );
+    VLC_CleanUp( 0 );
 
     /* Destroy the libvlc structure */
     VLC_Destroy( 0 );
@@ -113,7 +113,7 @@ int main( int i_argc, char *ppsz_argv[] )
     return i_ret;
 }
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(UNDER_CE)
 /*****************************************************************************
  * SigHandler: system signal handler
  *****************************************************************************
@@ -156,3 +156,32 @@ static void SigHandler( int i_signal )
 }
 #endif
 
+#if defined(UNDER_CE)
+#   if defined( _MSC_VER ) && defined( UNDER_CE )
+#       include "vlc_common.h"
+#   endif
+/*****************************************************************************
+ * WinMain: parse command line, start interface and spawn threads. (WinCE only)
+ *****************************************************************************/
+int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
+                    LPTSTR lpCmdLine, int nCmdShow )
+{
+    char **argv, psz_cmdline[MAX_PATH];
+    int argc, i_ret;
+
+    WideCharToMultiByte( CP_ACP, 0, lpCmdLine, -1,
+                         psz_cmdline, MAX_PATH, NULL, NULL );
+
+    argv = vlc_parse_cmdline( psz_cmdline, &argc );
+    argv = realloc( argv, (argc + 1) * sizeof(char *) );
+    if( !argv ) return -1;
+
+    if( argc ) memmove( argv + 1, argv, argc * sizeof(char *) );
+    argv[0] = ""; /* Fake program path */
+
+    i_ret = main( argc + 1, argv );
+
+    /* No need to free the argv memory */
+    return i_ret;
+}
+#endif
